/*  Ishape2d.geo
Geometrical description (for gmsh) of Ishape2D test for ECE
For details, see comments in the Ishape2d_data.pro file
Meshing information is also defined here.
*/

Include "Ishape2d_data.pro";

/* Definition of parameters for local mesh dimensions */
p0 = s*l/10; // characteristic length of mesh element

/* Definition of gemetrical points */
Point(1) = { -a, 0, 0, p0} ;
Point(2) = {  a, 0, 0, p0} ;
Point(3) = {  a, l, 0, p0} ;
Point(4) = { -a, l, 0, p0} ;

/* Definition of gemetrical lines */
Line(1) = {1,2};
Line(2) = {2,3};
Line(3) = {3,4};
Line(4) = {4,1};

/* Definition of geometrical surfaces */
Line Loop(5) = {1, 2, 3, 4};
Plane Surface(6) = {5};

If(_use_transfinite)
  Transfinite Line {2,4} = 3*s;
  Transfinite Line {1,3} = 1*s;
  Transfinite Surface {6};
EndIf

/* Definition of Physical entities (surfaces, lines). The Physical
   entities tell GMSH the elements and their associated region numbers
   to save in the file 'Ishape2d.msh'. */

Physical Surface ("MaterialX", 100) = {6} ;      /* MaterialX */

Physical Line ("Ground",   120) = {3} ;          /* Ground */
Physical Line ("Terminal", 121) = {1} ;          /* Terminal */
Physical Line ("RightBoundary", 131) = {2} ;     /* RightBoundary */
Physical Line ("LeftBoundary", 132) = {4} ;      /* LeftBoundary */
