/*  Ishape2d.geo
Geometrical description (for gmsh) of Ishape2D test for ECE
For details, see comments in the Ishape2d_data.pro file
Meshing information is also defined here.
The mesh size depends on the skin depth.
*/


Include "Ishape2d_data.pro";

/* Definition of parameters for local mesh dimensions */
//p0 = s*l/10; // characteristic length of mesh element
delta = Sqrt(2.0/(2*Pi*Freq*mu*sigma));
If (delta < a)
	p0 = delta/nbDelta;
Else
	p0 = l/10/2;
EndIf

/* Definition of gemetrical points */
Point(1) = { -a, 0, 0, p0} ;
Point(2) = {  a, 0, 0, p0} ;
Point(3) = {  a, l, 0, p0} ;
Point(4) = { -a, l, 0, p0} ;

/* Definition of gemetrical lines */
Line(1) = {1,2};
Line(2) = {2,3};
Line(3) = {3,4};
Line(4) = {4,1};

/* Definition of geometrical surfaces */
Line Loop(5) = {1, 2, 3, 4};
Plane Surface(6) = {5};

/* this is not used now
If(_use_transfinite)
  Transfinite Line {2,4} = 3*s;
  Transfinite Line {1,3} = 1*s;
  Transfinite Surface {6};
EndIf
*/

Field[1] = Distance;
Field[1].CurvesList = {2,4};
Field[1].Sampling = 50;
Field[2] = Threshold;
Field[2].InField = 1;
Field[2].SizeMin = p0;
Field[2].SizeMax = a/5;
Field[2].DistMin = 0;
Field[2].DistMax = 3*delta;


Field[3] = Min;
Field[3].FieldsList = {2};
Background Field = 3;
Mesh.MeshSizeExtendFromBoundary = 0;
Mesh.MeshSizeFromPoints = 0;
Mesh.MeshSizeFromCurvature = 0;
Mesh.Algorithm = 5;

/* Definition of Physical entities (surfaces, lines). The Physical
   entities tell GMSH the elements and their associated region numbers
   to save in the file 'Ishape2d.msh'. */

Physical Surface ("MaterialX", 100) = {6} ;      /* MaterialX */

Physical Line ("Ground",   120) = {3} ;          /* Ground */
Physical Line ("Terminal", 121) = {1} ;          /* Terminal */
Physical Line ("RightBoundary", 131) = {2} ;     /* RightBoundary */
Physical Line ("LeftBoundary", 132) = {4} ;      /* LeftBoundary */



