/*  Tshape2d.geo
Geometrical description (for gmsh) of Tshape2D test for ECE
For details, see comments in the Tshape2d_data.pro file
Meshing information is also defined here.
*/


Include "Tshape2d_data.pro";

/* Definition of parameters for local mesh dimensions */
p0 = s*l/10; // characteristic length of mesh element

/* Definition of gemetrical points */
Point(1) = { -a, 0, 0, p0} ;
Point(2) = {  a, 0, 0, p0} ;
Point(3) = {  a, l, 0, p0} ;
Point(4) = { -a, l, 0, p0} ;
Point(5) = { -a+b, 0, 0, p0} ;
Point(6) = { -a+b, c, 0, p0} ;
Point(7) = { a, c, 0, p0} ;
Point(8) = { a, c+d, 0, p0} ;
Point(9) = { -a+b, c+d, 0, p0} ;
Point(10) = { -a+b, l, 0, p0} ;

/* Definition of gemetrical lines */
Line(1) = {1,5};
Line(2) = {5,6};
Line(3) = {6,7};
Line(4) = {7,8};
Line(5) = {8,9};
Line(6) = {9,10};
Line(7) = {10,4};
Line(8) = {4,1};
Line(9) = {5,2};
Line(10) = {2,7};
Line(11) = {8,3};
Line(12) = {3,10};

/* Definition of geometrical surfaces */
Line Loop(13) = {1, 2, 3, 4, 5, 6, 7, 8};  // the T part
Line Loop(14) = {9, 10, -3, -2};
Line Loop(15) = {11, 12, -6, -5};
Plane Surface(16) = {13}; // the T part
Plane Surface(17) = {14}; // the non-conducting part
Plane Surface(18) = {15}; // the non-conducting part

/*
If(_use_transfinite)
  Transfinite Line {2,4} = 3*s;
  Transfinite Line {1,3} = 1*s;
  Transfinite Surface {6};
EndIf
*/

/* Definition of Physical entities (surfaces, lines). The Physical
   entities tell GMSH the elements and their associated region numbers
   to save in the file 'Tshape2d.msh'. */

Physical Surface ("Material1T", 100) = {16} ;             /* Material of the T part */
Physical Surface ("Material2Air", 101) = {17, 18} ;       /* Material of the rest */

Physical Line ("Ground",   120) = {1} ;        /* Ground */
Physical Line ("Terminal1", 121) = {7} ;       /* Terminal1 */
Physical Line ("Terminal2", 122) = {4} ;       /* Terminal2 */
Physical Line ("SurBCnotTerminal1", 201) = {8} ;      /* ECE boundary which is not terminal, touched by material1 */
Physical Line ("SurBCnotTerminal2", 202) = {9, 10, 11, 12} ;      /* ECE boundary which is not terminal, touched by material 2 */


