/*  Tshape2d.pro
Material description (for getdp) of Tshape2D test for ECE
For details, see comments in the Tshape2d_data.pro file
The Tshape2d geometry and meshing parameters are in the Tshape2d.geo file
This file contains the setings that depend on the particular problem and
it includes 1 general file
*/

Include "Tshape2d_data.pro"

Group{
  Material1T = Region[100];         /* MaterialX - the T shape part */
  Material2Air = Region[101];       /* Air - the rest */
  
  Ground = Region[120];    /* Boundary - various parts */
  Terminal1 = Region[121];
  Terminal2 = Region[122];
  
  SurBCnotTerminal1 = Region[201];
  SurBCnotTerminal2 = Region[202];
  
  
  Vol_FW = Region[{Material1T,Material2Air}]; // domain without the boundary
  Sur_Terminals_FWece = Region[{Ground, Terminal1, Terminal2}]; // all terminals
  Sur_FW = Region[{Sur_Terminals_FWece, SurBCnotTerminal1, SurBCnotTerminal2}]; //all boundary
  //Dom_FW = Region[ {Vol_FW, SurBCnotTerminal1, SurBCnotTerminal2} ];
  Dom_FW = Region[ {Vol_FW, Sur_FW} ];

 }
 
 
 Function{
   /* Material properties are defined piecewise, for elementary groups */
   /* the values in the right hand side are defined in Ishape2d_data.pro */
   
   nu[#{Material1T,SurBCnotTerminal1,Ground,Terminal1,Terminal2}] = nu1;             // magnetic reluctivity [m/H]
   epsilon[#{Material1T,SurBCnotTerminal1,Ground,Terminal1,Terminal2}] = eps1;       // permittivity [F/m]
   sigma[#{Material1T,SurBCnotTerminal1,Ground,Terminal1,Terminal2}]  = sigma1;      // conductivity [S/m]
   
   nu[#{Material2Air,SurBCnotTerminal2}] = nu2;             // magnetic reluctivity [m/H]
   epsilon[#{Material2Air,SurBCnotTerminal2}] = eps2;       // permittivity [F/m]
   sigma[#{Material2Air,SurBCnotTerminal2}]  = sigma2;      // conductivity [S/m]

   // This is a problem with 3 terminals, out of which one is grounded.
   // The non-grounded terminals can be excited in voltage or in current, but only one is active.
   // Depending on the excitation type, one of the following functions will be useful.
   //
   // When imposing voltage or current via a constraint, the region appears there
   // If you have a complex, the value has to be a function => use square brackets []
   If((Flag_AnalysisType==0)||(Flag_AnalysisType==1))
		VTerminal1[] = -VTerm1RMS*Complex[Cos[VTerm1Phase],Sin[VTerm1Phase]];
   Else
		ITerminal1[] = -ITerm1RMS*Complex[Cos[ITerm1Phase],Sin[ITerm1Phase]];
   EndIf
   If((Flag_AnalysisType==0)||(Flag_AnalysisType==2))
		VTerminal2[] = -VTerm2RMS*Complex[Cos[VTerm2Phase],Sin[VTerm2Phase]];
   Else
		ITerminal2[] = -ITerm2RMS*Complex[Cos[ITerm2Phase],Sin[ITerm2Phase]];
   EndIf
   
 }
 
 
 Constraint{
   { Name SetTerminalPotential; Type Assign;  // voltage excited terminals
     Case {
       { Region Ground; Value 0.; }
       If((Flag_AnalysisType==0)||(Flag_AnalysisType==1))
         { Region Terminal1; Value VTerminal1[]; }
       EndIf
	   If((Flag_AnalysisType==0)||(Flag_AnalysisType==2))
         { Region Terminal2; Value VTerminal2[]; }
       EndIf
     }
   }
   { Name SetTerminalCurrent; Type Assign;   // current excited terminals
     Case {
       If((Flag_AnalysisType==2)|| (Flag_AnalysisType==3))
         { Region Terminal1; Value ITerminal1[]/h2Ddepth; }  // here the depth is needed
       EndIf
	   If((Flag_AnalysisType==1)|| (Flag_AnalysisType==3))
         { Region Terminal2; Value ITerminal2[]/h2Ddepth; }  // here the depth is needed
       EndIf
     }
   }
}

 /* The formulation and its tools */
 Include "../problemIndependent_proFiles/FullWave_E_ece_MIMO2terminals.pro"

  If (Flag_AnalysisType==0)
	Dir = "res/FWeceBC_ev_ev/";
 ElseIf (Flag_AnalysisType==1)
	Dir = "res/FWeceBC_ev_ec/";
 ElseIf (Flag_AnalysisType==2)
	Dir = "res/FWeceBC_ec_ev/";
 Else
	Dir = "res/FWeceBC_ec_ec/";
 EndIf
 /* Output results */
Macro Change_post_options
Echo[Str[ "nv = PostProcessing.NbViews;",
    "For v In {0:nv-1}",
    "View[v].NbIso = 25;",
    "View[v].RangeType = 3;" ,// per timestep
    "View[v].ShowTime = 3;",
    "View[v].IntervalsType = 3;",
    "EndFor"
  ], File "post.opt"];
Return

If ((TypeExcTerminal1==1)&(TypeExcTerminal2==1))  // current exc
	snpfilename1 = StrCat[Dir, Sprintf("test_Z1%g_RI.partial_s2p.s1p",ActiveTerminal)];
	snpfilename2 = StrCat[Dir, Sprintf("test_Z2%g_RI.partial_s2p.s1p",ActiveTerminal)];
ElseIf ((TypeExcTerminal1==2)&(TypeExcTerminal2==2))  // voltage exc
    snpfilename1 = StrCat[Dir, Sprintf("test_Y1%g_RI.partial_s2p.s1p",ActiveTerminal)];
	snpfilename2 = StrCat[Dir, Sprintf("test_Y2%g_RI.partial_s2p.s1p",ActiveTerminal)];
ElseIf ((TypeExcTerminal1==1)&(TypeExcTerminal2==2))  //hybrid H exc
    snpfilename1 = StrCat[Dir, Sprintf("test_H1%g_RI.partial_s2p.s1p",ActiveTerminal)];
	snpfilename2 = StrCat[Dir, Sprintf("test_H2%g_RI.partial_s2p.s1p",ActiveTerminal)];
Else  //hybrid G exc
    snpfilename1 = StrCat[Dir, Sprintf("test_G1%g_RI.partial_s2p.s1p",ActiveTerminal)];
	snpfilename2 = StrCat[Dir, Sprintf("test_G2%g_RI.partial_s2p.s1p",ActiveTerminal)];
EndIf

Macro WriteZ_header
  Echo[ Str[Sprintf("# Hz Z RI R %g", 50)], Format Table, File > snpfilename1 ];
  Echo[ Str[Sprintf("# Hz Z RI R %g", 50)], Format Table, File > snpfilename2 ];
Return
Macro WriteY_header
  Echo[ Str[Sprintf("# Hz Y RI R %g", 50)], Format Table, File > snpfilename1 ];
  Echo[ Str[Sprintf("# Hz Y RI R %g", 50)], Format Table, File > snpfilename2 ];
Return
Macro WriteH_header
  Echo[ Str[Sprintf("# Hz H RI R %g", 50)], Format Table, File > snpfilename1 ];
  Echo[ Str[Sprintf("# Hz H RI R %g", 50)], Format Table, File > snpfilename2 ];
Return
Macro WriteG_header
  Echo[ Str[Sprintf("# Hz G RI R %g", 50)], Format Table, File > snpfilename1 ];
  Echo[ Str[Sprintf("# Hz G RI R %g", 50)], Format Table, File > snpfilename2 ];
Return

 PostOperation {
  { Name TransferMatrix; NameOfPostProcessing FW_E_ece ;
     Operation {
       If(Flag_AnalysisType==3)  // current excitation
         If (Freq==freqs(0))
           Call WriteZ_header;
           Printf("======================> Writting header in _partial files (Z) with first Freq=%g",Freq);
         EndIf
		 Printf("****************** Freq=%g ******************",Freq);
         Print [ Vterminal1, OnRegion Terminal1,  
           Format Table , File  > snpfilename1] ;
		 Print [ Vterminal2, OnRegion Terminal2,  
           Format Table , File  > snpfilename2] ;  
       ElseIf(Flag_AnalysisType==0)  // voltage excitation
         If (Freq==freqs(0))
           Call WriteY_header;
           Printf("======================> Writting header in _partial files (Y) with first Freq=%g",Freq);
         EndIf
		 Printf("****************** Freq=%g ******************",Freq);
		 Print [ Iterminal1, OnRegion Terminal1,  
           Format Table , File  > snpfilename1] ;
		 Print [ Iterminal2, OnRegion Terminal2,  
           Format Table , File  > snpfilename2] ;  
	    ElseIf(Flag_AnalysisType==2)  // hybrid excitation (first ec, second ev)
         If (Freq==freqs(0))
           Call WriteH_header;
		   Printf("======================> Writting header in _partial files (H) with first Freq=%g",Freq);
         EndIf
		 Printf("****************** Freq=%g ******************",Freq);
         Print [ Vterminal1, OnRegion Terminal1,  
           Format Table , File  > snpfilename1] ;
		 Print [ Iterminal2, OnRegion Terminal2,  
           Format Table , File  > snpfilename2] ;  
	     Else // reverse hybrid excitation (first ev, second ec)
			If (Freq==freqs(0))
				Call WriteG_header;
				Printf("======================> Writting header in _partial files (G) with first Freq=%g",Freq);
			EndIf
		 Printf("****************** Freq=%g ******************",Freq);
         Print [ Iterminal1, OnRegion Terminal1,  
           Format Table , File  > snpfilename1] ;
		 Print [ Vterminal2, OnRegion Terminal2,  
           Format Table , File  > snpfilename2] ;  
	     EndIf
     }
   }

 
  { Name Maps; NameOfPostProcessing FW_E_ece ;
     Operation {
       Print [ gradV, OnElementsOf ElementsOf[Vol_FW,OnOneSideOf Sur_FW], File StrCat[Dir,"map_gradV.pos" ]]; //_v4
       Print [ E, OnElementsOf Vol_FW, File StrCat[Dir,"map_E.pos" ]];
	   Print [ J, OnElementsOf Vol_FW, File StrCat[Dir,"map_J.pos" ]];

       Print [ H, OnElementsOf Vol_FW, File StrCat[Dir,"map_H.pos" ]];
       Print [ Vterminals, OnElementsOf ElementsOf[Sur_FW], File StrCat[Dir,"map_Vterminals.pos" ]];
	   Print [ VnotTerminals, OnElementsOf Vol_FW, File StrCat[Dir,"map_VnotTerminals.pos" ]];


       Call Change_post_options;
     }
  }
  
}
