DefineConstant[
  cm = 1e-2,
  mm = 1e-3,
  nturns = {8, Name "Param. Geometry/00Nb coil turns", Highlight "Pink"}
  //nturns = {2, Name "Param. Geometry/00Nb coil turns", Highlight "Pink"}
  lengthInd = {9*cm, Name "Param. Geometry/01length of inductor", Highlight "Pink"}
  rin = {17*mm, Name "Param. Geometry/02inner radius inductor", Highlight "Pink"}
  rw     = {5*mm/2, Name "Param. Geometry/03radius or wire", Highlight "Pink"}
  pitch  = {lengthInd/(nturns-1), Name "Param. Geometry/04pitch", Highlight "Pink", ReadOnly 1}
  rplateC = {6*cm, Name "Param. Geometry/10radius of capacitor plates", Highlight "LightCyan"}
  dplateC = {3*mm, Name "Param. Geometry/11distance between plates", Highlight "LightCyan"}
];

dCL = lengthInd;

sigma_cond = 1e7;
mur_core = 1000;
epsr_diel = 1e6;
sigma_diel = 0;

Rext = 450*1e-3;

mu0 = 4.e-7 * Pi ;
nu0  = 1/mu0;
eps0 = 8.854187818e-12;

// *****************

nuAir = nu0;
epsAir = eps0;
sigmaAir = 0;

nuCore = 1/(mu0*mur_core);
epsCore = eps0;
sigmaCore = 0;

nuDiel = nu0;
epsDiel = eps0*epsr_diel;
sigmaDiel = sigma_diel;

nuCond = nu0;
epsCond = eps0;
sigmaCond = sigma_cond;

// ******************

// Parallel plate capacitor with C
Canalytical = DefineNumber[ epsr_diel * eps0 * (Pi*rplateC^2)/dplateC*1e6, ReadOnly 1,
  Name "Param. Geometry/90Cana (planar capacitor)", Units "µF", Highlight "MidnightBlue"];

Lanalytical_solenoid = DefineNumber[ 1e6*nturns^2 * mu0* (Pi*(rin+rw)^2)/lengthInd, ReadOnly 1,
  Name "Param. Geometry/91Lana (air core)", Units "µH", Highlight "ForestGreen"];
K = 1+0.9*(rin+rw)/lengthInd;
L_Nagaoka = nturns^2*mu0*(Pi*(rin+rw)^2)/(lengthInd*K);
Lanalytical_solenoid2 = DefineNumber[ 1e6*L_Nagaoka, ReadOnly 1,
  Name "Param. Geometry/91Lana2 (air core)", Units "µH", Highlight "ForestGreen"];


// Stray capacitance and self capacitance....
// IEEE TRANSACTIONS ON INDUSTRY APPLICATIONS, VOL. 35, NO. 5, SEPTEMBER/OCTOBER 1999
epsr=1;
D = 2*(rin+rw); //turn diameter
// turn-to-turn capacitance
Ctt = Pi^2*D*epsr*eps0/Log[ pitch/(2*rw) + Sqrt[(pitch/(2*rw))^2-1] ];
// turn-to-shield capacitance
hsc = dCL;
Cts = 2*Pi^2*D*epsr*eps0/Log[ hsc/rw + Sqrt[(hsc/rw)^2-1]];

Ctot = Ctt/(nturns-1);
Printf("Ctot %g F", Ctot);

CAB~{2} = Ctt+Cts/2;
CAB~{3} = Ctt/2+Cts/2;
For k In {4:nturns:1}
  CAB~{k} = (CAB~{k-2}*Ctt/2)/(CAB~{k-2}+Ctt/2)+Cts/2;
Printf("CAB_%g %g", k, CAB~{k});
EndFor
Printf("Ctt %g F Cts %g F", Ctt, Cts);
Ctot = CAB~{nturns}/(nturns-1);
Printf("Ctot %g F", Ctot);

// IEEE Trans. PE, VOL. 34, NO. 11, Nov. 2019
aiso=pitch;
di=2*rw;
de=di;
hiso= pitch;
lmlt = 2*Pi*(rin+rw);// mean length per turn of the winding
rww = lmlt/(2*Pi);
hww = lengthInd;
dww = aiso-0.15*di+0.26*(hiso+di);
Cww = epsr*eps0* 2*Pi*hww*rww/dww;
Printf("Cww %g F", Cww);
Ctot = Cww/(nturns-1);
Printf("Ctot %g F", Ctot);

T=0*(de-di);
alpha=1-T/(epsr*di);
beta = 1/alpha*(1+pitch/(2*epsr*rw));
Y = beta/Sqrt[beta^2-1]*Atan[Sqrt[(beta+1)/(beta-1)]]-Pi/4;
Z = beta*(beta^2-2)/(beta^2-1)^1.5*Atan[Sqrt[(beta+1)/(beta-1)]]-0.5*beta/(beta^2-1)-Pi/4;

Cij=2*eps0*lmlt/alpha*(Y+1/(8*epsr)*(2*T/rw)^2*Z/alpha);
Printf("Cij %g F", Cij);

// Inductance straight round wire in [nH]
// Lwire = 2*L*ln[(2*L/D)*(1+Sqrt[1+(D/(2*L)^2)])] - Sqrt[1+(D/(2*L)^2)] + mu/4+(D/2*L);
// D = diameter of the wire in cm
// L = length of the wire cm
// Note: from “Inductance Calculations” by F. W. Grover

// lw = (3*dCL+3*dplateC)/cm; // If LC
// lw = (2*dCL)/cm; // If L
// dw =2*rw/cm;
// x = Sqrt[1+(dw/(2*lw))^2];
// Lwire = DefineNumber[ 2*lw*(Log[(2*lw/dw)*(1+x)]-x+mu0/4+dw/(2*lw)), ReadOnly 1,
//   Name "Param. Geometry/91Analytical Lwire", Units "nH", Highlight "ForestGreen"];
// Ltot = DefineNumber[Lwire*1e-6 + Lanalytical_solenoid, ReadOnly 1,
//   Name "Param. Geometry/92Ltot", Units "mH", Highlight "MidnightBlue"];


// Physical numbers...
//----------------------
CONDUCTOR     = 1000;  // volume, conductor touching the boundary at the excited termimnal, including the coil wire and one plate
BND_CONDUCTOR = 1111;  // volume, conductor touching the boundary at the grounded terminal and including the other plate

ELEC_IN  = 111; // surface, excited terminal
ELEC_OUT = 112; // surface, grounded terminal

DIELECTRIC   = 2000; // volume, the dielectric of the capacitor
ELEC_DIEL_IN = 221;
ELEC_DIEL_OUT = 222;

CORE = 3000;  // volume, the core of the coil

AIR = 4000;  // volume, the air
BND_AIR = 4444; // surface, boundary touched by air

// **********************
cGUI= "{CL - Excitation/";
close_menu = 1;
colorro = "LightGrey";
mTypeBC = StrCat[cGUI,"Type of ECE Excitation/0"];

colorMTypeBC = "Red";
DefineConstant[
  Flag_AnalysisType = { 0,
    Choices{
      0="ev",
      1="ec"},
    Name Str[mTypeBC], Highlight Str[colorMTypeBC], Closed !close_menu,
    ServerAction Str["Reset", "GetDP/1ResolutionChoices"]}
];

/* new menu for the interface - values of excitations - significance depends on the type of BC */
mValuesBC = StrCat[cGUI,"Frequency values/0"];
colorMValuesBC = "Ivory";

fmin = 1000;   // Hz
fmax = 80000; // Hz

//freqs()= LogSpace[7,11,5];
nop = 10;
//freqs()=LogSpace[Log10[fmin],Log10[fmax],nop];
freqs()=LinSpace[fmin,fmax,nop];
DefineConstant[
  Freq  = {freqs(0), Choices{freqs()}, Loop, Name StrCat[mValuesBC, "0Working Frequency"], Units "Hz", Highlight  Str[colorMValuesBC], Closed !close_menu }
];

//DefineConstant[
  // Flag_AnalysisType
  // ==0 bottom ev
  // ==1 bottom ec

  //VGroundRMS  = {0, Name StrCat[mValuesBC, "2Ground/1Potential (rms)"], Units "V" , ReadOnly 1, Highlight Str[colorro],
  //  Visible (Flag_AnalysisType==0) || (Flag_AnalysisType==1)}
  //VGroundPhase  = {0, Name StrCat[mValuesBC, "2Ground/1Potential, phase"], Units "rad",  ReadOnly 1, Highlight Str[colorro],
  //  Visible (Flag_AnalysisType==0) || (Flag_AnalysisType==1)}	
//];


If (Flag_AnalysisType==1) // current exc
	Printf("======================> Terminal is current => Z transfer function");
	ITermRMS = 1; 
	ITermPhase = 0;
Else  // voltage exc
	Printf("======================> Yerminal is voltage excited => Y transfer function");
	VTermRMS = 1; 
	VTermPhase = 0;
EndIf

modelDim = 3; // 
Flag_Axi = 0; // 1 for AXI - it makes sense only for modelDim = 2

If ((modelDim == 2)&&(Flag_Axi == 0))
	h2Ddepth = h;
ElseIf ((modelDim == 2)&&(Flag_Axi == 1))   // 2D AXI
	h2Ddepth = 2*Pi;
Else // 3D
    h2Ddepth = 1;
EndIf