/* Ishape2d_data.pro
=========================== Ishape2D - test problem
A rectangular domain in the plane xOy, covering [-a,a] X [0,l]
(width = 2a, height = l);
A depth h (dimension along Oz) is needed for postrocessing or for the current
excitation case.
The domain is filled with homogeneous and linear material, with epsilon, mu, sigma.
There is no no field source inside the domein (the "device" is passive).
The field sources are only boundary conditions (BC) which can be eiter (exclusive or)
1) classical BC - Et and Ht given in any point of the boundary
2) ECE BC - surface satisfies ECE BC, meaning that only the position of terminals has to
be given, their excitation type (voltage xor current) and their excitation signals
(frequency domain simulations, i.e. root means square values and initial phases have to
be given for the input signals)

This is a test problem - used for the first validation of the ECE implementation in onelab.
As an ECE BC, it is a device with 2 terminals, out of which one is ground.
Thus, as a system this is a single input single output (SISO) system.

===========================
Names of input data - to be given by the user
Geometry:
a = half of the dimension on the Ox axis [m]
l = dimension along the Oy axis [m]
h = dimension along the Oz axis (useful for postprocessing and for current excited case) [m]
==
Material:
epsr = relative permittivity
mur = relative permeability
sigma = conductivity [S/m
==
Type of BC:
typeBC = 0 (ECE - voltage exc),1 (crt exc)

=========================== */


NOPARAMS = 0; // set to 0 when calling from Matlab 

// geometry
a = 2.5e-6;
l = 10e-6;
h = 10e-6;

// material
epsr = 1;
mur = 1;
sigma = 6.6e7;


eps0 = 8.854187818e-12; // F/m - absolute permitivity of vacuum
mu0  = 4.e-7 * Pi;      // H/m - absolute permeability of vacuum
nu0  = 1/mu0;           // m/H - absolute reluctivity of vacuum

eps = eps0*epsr;        // F/m - absolute permitivity
mu = mu0*mur;           // F/m - absolute permeability
nu = 1/mu;              // m/H - absolute reluctivity

/* new menu for the interface - type of BC */
cGUI= "{Ishape2D-ProblemData/";
close_menu = 1;
mTypeBC = StrCat[cGUI,"Type of BC/0"];
colorMTypeBC = "Red";
DefineConstant[
  Flag_AnalysisType = { 0,
    Choices{
      0="ECE: voltage excitation on bottom, ground on top",
      1="ECE: current excitation on bottom, ground on top"
    },
    Name Str[mTypeBC], Highlight Str[colorMTypeBC], Closed !close_menu,
    Help Str["s1p files will be saved in res folder"],
    ServerAction Str["Reset", "GetDP/1ResolutionChoices"]}
];


// frequencies will be sent from Matlab (Freq and freqs)

// What is used depends on the Flag_AnalysisType: 0 for ev, 1 for ec
VBottomRMS = 	1;
VBottomPhase = 0;

IBottomRMS = 1;
IBottomPhase = 0;

  
VtTopRMS   = 0;
VtTopPhase = 0;
 
//mesh  
_use_transfinite = 1;
s = 30; //10

modelDim = 2; // 
Flag_Axi = 0; // 1 for AXI it makes sense only for modelDim = 2

If ((modelDim == 2)&&(Flag_Axi == 0))
	h2Ddepth = h;
ElseIf ((modelDim == 2)&&(Flag_Axi == 1))   // 2D AXI
	h2Ddepth = 2*Pi;
Else // 3D
    h2Ddepth = 1;
EndIf



