function [transferFunction] = auto_vfit3_fromfreqdata(...
    frequency_data,response,snp_info,avfitParams)
% Input:
%
% frequency_data.frequency_unit
% frequency_data.frequency_points
%
% response
%
% snp_info.ptype
% snp_info.pformatfile
% snp_info.Z0
% snp_info.nports
%
% avfitParams.minOrder = 1;
% avfitParams.maxOrder = 10;
% avfitParams.tol = 1e-2;
% avfitParams.noCalls = 2;
% avfitParams.typeTransferFunction = 2; %1 for Strictly Proper, 2 for Proper, 3 for Improper
% avfitParams.graphicsVfit = 'no'; %'yes', otherwise treated as 'no'
% avfitParams.stopCriterion = 0; %0 - components for what it is (codestar way), 1 - Frobenius S, 2 - Components S
% avfitParams.fileout = 'test.cir';
% avfitParams.idx = 0; % useful only when combined with sys2snp

% transferFunction.order
%                 .poles
%                 .residues
%                 .kinf
%                 .prop
%%%

fmin = frequency_data.frequency_points(1);
nof = length(frequency_data.frequency_points);
fmax = frequency_data.frequency_points(nof);
nop = snp_info.nports;
if strcmpi(frequency_data.frequency_unit,'HZ')
    frad = 2*pi*frequency_data.frequency_points;
elseif strcmpi(frequency_data.frequency_unit,'KHZ')
    %disp('change frequency from KHz to rad/s')
    frad = 2*pi*frequency_data.frequency_points*1e3;
elseif strcmpi(frequency_data.frequency_unit,'MHZ')
    %disp('change frequency from MHz to rad/s')
    frad = 2*pi*frequency_data.frequency_points*1e6;
elseif strcmpi(frequency_data.frequency_unit,'GHZ')
    %disp('change frequency from GHz to rad/s');
    frad = 2*pi*frequency_data.frequency_points*1e9;
else
    frad = frequency_data.frequency_points;
    %disp('frequency is in rad/s');
end;
s = frad*1i;

if strcmpi(avfitParams.save.type,'lin')
    frecv2 = linspace(fmin,fmax,avfitParams.save.nopoints);
else
    frecv2 = logspace(log10(fmin),log10(fmax),avfitParams.save.nopoints);
end
if strcmpi(frequency_data.frequency_unit,'HZ')
    frecv2r = 2*pi*frecv2;
elseif strcmpi(frequency_data.frequency_unit,'KHZ')
    %disp('change frequency from KHz to rad/s')
    frecv2r = 2*pi*frecv2*1e3;
elseif strcmpi(frequency_data.frequency_unit,'MHZ')
    %disp('change frequency from MHz to rad/s')
    frecv2r = 2*pi*frecv2*1e6;
elseif strcmpi(frequency_data.frequency_unit,'GHZ')
    %disp('change frequency from GHz to rad/s');
    frecv2r = 2*pi*frecv2*1e9;
else
    frecv2r = frecv2;
    %disp('frequency is in rad/s');
end;


er = 10;
ordin = avfitParams.minOrder;
if nof <= avfitParams.maxOrder
    disp(sprintf('MAXORDER FOR AUTO VF3 HAS CHANGED TO %d',nof-1));
    avfitParams.maxOrder = nof - 1;
end
max_no_of_trials = avfitParams.maxOrder - avfitParams.minOrder + 1;
ord = zeros(1,max_no_of_trials);
erord = zeros(1,max_no_of_trials);
ok1 = 0;
ok2 = 0;
idx_trial = 0;

if length(size(response)) == 2 %SISO
    nopct = max(size(response));
    newresp = zeros(1,1,nopct);
    newresp(1,1,:) = response;
else
    newresp = response;
end

if avfitParams.typeTransferFunction ==1  %1 for Strictly Proper, 2 for Proper, 3 for Improper
    
    opts.asymp=1; % --> D=0,  E=0  ('Strictly proper')
elseif avfitParams.typeTransferFunction ==2
    opts.asymp=2; % --> D~=0, E=0  ('Proper')
else %(Def=2)
    opts.asymp=3; % --> D~=0, E~=0 ('Improper')
end

while ((er > avfitParams.tol) && (ordin <= avfitParams.maxOrder) && ...
        (ordin >= avfitParams.minOrder) && (ok1 == 0) && (ok2 == 0))
    
    opts.N = ordin;              %Order of approximation. (Is used when opts.poles=[]).
    opts.poletype='lincmplx';   %Will use logarithmically spaced, complex poles. (Is used when opts.poles=[]).
    opts.plot = 0; % ca sa nu mai deseneze pe ecran
    opts.logx = 0; % ca sa afiseze scara liniara pe frecventa
    opts.screen = 0; % ca sa nu mai scrie in consola
    if strcmpi(avfitParams.graphicsVfit,'yes')
        opts.plot = 1;
    end
    
 


    
    poles=[]; %[] -->N initial poles are automatically generated as defined by opts.startpoleflag
    [SER,rmserr,bigYfit,opts2]=VFdriver(newresp,s,poles,opts); %Creating state-space model and pole-residue model
    poles = SER.poles;
    residues = SER.R;
    kinf = SER.D;
    prop = SER.E;
    
    if ok1 == 0
        if nop == 1 %SISO
            response_red = zeros(nof,1);
            for k1 = 1:nof
                tf = 0;
                omega_crt = frad(k1);
                for k2 = 1:ordin
                    tf = tf + residues(k2)/(1i*omega_crt - poles(k2));
                end
                tf = tf + kinf + 1i*omega_crt*prop;
                response_red(k1) = tf;
            end
        else
            response_red = zeros(nop,nop,nof);
            for k1 = 1:nof
                tf = zeros(nop,nop);
                omega_crt = frad(k1);
                for k2 = 1:ordin
                    tf = tf + residues(:,:,k2)./(1i*omega_crt - poles(k2));
                end
                tf = tf + kinf + 1i*omega_crt*prop;
                response_red(:,:,k1) = tf;
            end
        end
        
        if strcmpi(avfitParams.save.flag,'yes') % computes vfit approximation in many points
            
            if nop == 1 %SISO
                rasp2 = zeros(avfitParams.save.nopoints,1);
                for k1 = 1:avfitParams.save.nopoints
                    tf = 0;
                    omega_crt = frecv2r(k1);
                    for k2 = 1:ordin
                        tf = tf + residues(k2)/(1i*omega_crt - poles(k2));
                    end
                    tf = tf + kinf + 1i*omega_crt*prop;
                    rasp2(k1) = tf;
                end
            else
                rasp2 = zeros(nop,nop,avfitParams.save.nopoints);
                for k1 = 1:avfitParams.save.nopoints
                    tf = zeros(nop,nop);
                    omega_crt = frecv2r(k1);
                    for k2 = 1:ordin
                        tf = tf + residues(:,:,k2)./(1i*omega_crt - poles(k2));
                    end
                    tf = tf + kinf + 1i*omega_crt*prop;
                    rasp2(:,:,k1) = tf;
                end
            end
            
            
            writesnp_v2(strcat('vfit_interm_iter',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'), ...
                frecv2, rasp2, snp_info.ptype, ...
                frequency_data.frequency_unit,snp_info.Z0,'RI');
            if strcmpi(avfitParams.save.flagMA,'yes')
                writesnp_v2(strcat('vfit_MA_interm_iter',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'), ...
                    frecv2, rasp2, snp_info.ptype, ...
                    frequency_data.frequency_unit,snp_info.Z0,'MA');
            end
            if strcmpi(avfitParams.save.flagMAinDB,'yes')
                writesnp_v2(strcat('vfit_MAinDB_interm_iter',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'), ...
                    frecv2, rasp2, snp_info.ptype, ...
                    frequency_data.frequency_unit,snp_info.Z0,'DB');
            end
            
            writesnp_v2(strcat('vfit_interm_points',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'),...
                frequency_data.frequency_points, response, snp_info.ptype, ...
                frequency_data.frequency_unit,snp_info.Z0,'RI');
            if strcmpi(avfitParams.save.flagMA,'yes')
                writesnp_v2(strcat('vfit_MA_interm_points',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'),...
                    frequency_data.frequency_points, response, snp_info.ptype, ...
                    frequency_data.frequency_unit,snp_info.Z0,'MA');
            end
            if strcmpi(avfitParams.save.flagMAinDB,'yes')
                writesnp_v2(strcat('vfit_MAinDB_interm_points',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'),...
                    frequency_data.frequency_points, response, snp_info.ptype, ...
                    frequency_data.frequency_unit,snp_info.Z0,'DB');
            end
            
        end
        ll = length(response_red);
        er = compute_error(nop,ll,snp_info.ptype,response,response_red,...
            snp_info.Z0,snp_info.Z0,avfitParams.stopCriterion);
        fprintf('order %d, error is %e \n',ordin,er);
        idx_trial = idx_trial + 1;
        ord(idx_trial) = ordin;
        erord(idx_trial) = er;
        ordin = ordin + 1;
        
    end
end
ord(idx_trial+1:max_no_of_trials) = [];
erord(idx_trial+1:max_no_of_trials) = [];
ordin = ordin - 1;
%%%
if and(ok1 == 0, ok2 == 0)
    % check and enforce passivity
    clear opts;
    opts.parametertype='Y';
    %opts.plot.s_pass=2*pi*1i*linspace(1e9,40e9,1001).';
    %opts.plot.ylim=[-2e-3 2e-3];
    [SER,bigYfit_passive,opts3]=RPdriver(SER,s,opts);
    save('SER14.mat','SER');
    
    % calcul final in multe puncte
    poles = SER.poles;
    residues = SER.R;
    kinf = SER.D;
    prop = SER.E;
    if nop == 1 %SISO
        rasp2 = zeros(avfitParams.save.nopoints,1);
        for k1 = 1:avfitParams.save.nopoints
            tf = 0;
            omega_crt = frecv2r(k1);
            for k2 = 1:ordin
                tf = tf + residues(k2)/(1i*omega_crt - poles(k2));
            end
            tf = tf + kinf + 1i*omega_crt*prop;
            rasp2(k1) = tf;
        end
    else
        rasp2 = zeros(nop,nop,avfitParams.save.nopoints);
        for k1 = 1:avfitParams.save.nopoints
            tf = zeros(nop,nop);
            omega_crt = frecv2r(k1);
            for k2 = 1:ordin
                tf = tf + residues(:,:,k2)./(1i*omega_crt - poles(k2));
            end
            tf = tf + kinf + 1i*omega_crt*prop;
            rasp2(:,:,k1) = tf; %reshape(tf,nop,nop);
        end
    end
    
    
    writesnp_v2(strcat('vfit_final_iter',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'), ...
        frecv2, rasp2, snp_info.ptype, ...
        frequency_data.frequency_unit,snp_info.Z0,'RI');
    if strcmpi(avfitParams.save.flagMA,'yes')
        writesnp_v2(strcat('vfit_MA_final_iter',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'), ...
            frecv2, rasp2, snp_info.ptype, ...
            frequency_data.frequency_unit,snp_info.Z0,'MA');
    end
    if strcmpi(avfitParams.save.flagMAinDB,'yes')
        writesnp_v2(strcat('vfit_MAinDB_final_iter',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'), ...
            frecv2, rasp2, snp_info.ptype, ...
            frequency_data.frequency_unit,snp_info.Z0,'DB');
    end
    
    writesnp_v2(strcat('vfit_final_points',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'),...
        frequency_data.frequency_points, response, snp_info.ptype, ...
        frequency_data.frequency_unit,snp_info.Z0,'RI');
    if strcmpi(avfitParams.save.flagMA,'yes')
        writesnp_v2(strcat('vfit_MA_final_points',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'),...
            frequency_data.frequency_points, response, snp_info.ptype, ...
            frequency_data.frequency_unit,snp_info.Z0,'MA');
    end
    if strcmpi(avfitParams.save.flagMAinDB,'yes')
        writesnp_v2(strcat('vfit_MAinDB_final_points',num2str(avfitParams.idx),'_ord',num2str(length(poles)),'.s',num2str(nop),'p'),...
            frequency_data.frequency_points, response, snp_info.ptype, ...
            frequency_data.frequency_unit,snp_info.Z0,'DB');
    end
    
    switch avfitParams.stopCriterion
        case 2
            fprintf('# order error:max rms Components S #');
        case 1
            fprintf('# order error: Frobenius S # \n');
        otherwise
            fprintf('# order error: Frobenius %s # \n',snp_info.ptype);
    end
    for i = 1:length(ord)
        fprintf('%d \t %e \n',ord(i),erord(i));
    end
    disp('# auto vfit - NORMAL STOP');
    out = ord(length(ord));
    transferFunction.order = out;
    transferFunction.poles = poles;
    transferFunction.residues = residues;
    transferFunction.kinf = kinf;
    transferFunction.prop = prop;
    disp('# Carry out the synthesis ');
    ok2 = exportDEM2(poles,residues,kinf,avfitParams.fileout,prop,snp_info.ptype,fmin,fmax,'YES');
else
    disp('# auto vfit - FAILURE');
    transferFunction.order = -1;
    transferFunction.poles = [];
    transferFunction.residues = [];
    transferFunction.kinf = [];
    transferFunction.prop = [];
end

end