function [frequency_response,frequency_data,trfct] = ...
    afs_vf_onelab(frequency_data,snp_info,avfitParams,Flag_AnalysisType)  % old sys2snp_vf3
%
% afs implementat nerecursiv
% adapted for use with onelab function from the old vile sys2snp_vf3,m
% it uses autovectorfit to interpolate the answer in an intermediate point
% ==
%
% Input:
%        frequency_data - structure with the frequency information
%             frequency_data.frequency_unit - it can be 'Hz', 'KHz', 'MHz',
%                               'GHz', otherwise it is considered 'rad/sec'
%             frequency_data.frequency_points - vector with frequencies
%                               if afs flag is false, the answer will be
%                               computed exactly for these frequencies,
%                               otherwise, only the first and the last one
%                               are used as frequency range
%             frequency_data.AFS.flag  - boolean flag, AFS will be used
%                                        only if this flag is true
%                               .max   - max no of iterations for afs
%                               .err   - rel error for afs
%                               .type - 'lin' for linear interpolation
%                                        otherwise polytropic
%
%         method - string of characters, it can be only 'onelab' here:
%
% Output - frequency_response - the output quantity
%          frequency_data - updated frequency (it is different from input
%                           quantity only if afs was used
%
% Last update: Gabriela Ciuprina, 22 Octomber 2021

trfct = [];

m = snp_info.nports; % no of terminals
nofreq = length(frequency_data.frequency_points); % no of frequency points

% check if the frequencies are in rad/sec
switch lower(frequency_data.frequency_unit)
    case 'hz'
        freqs = frequency_data.frequency_points;
    case 'khz'
        freqs = frequency_data.frequency_points*1e3;
    case 'mhz'
        freqs = frequency_data.frequency_points*1e6;
    case 'ghz'
        freqs = frequency_data.frequency_points*1e9;
    otherwise % rad/sec
        freqs = frequency_data.frequency_points/(2*pi);
end

t1 = cputime;
%%%%%%%%%%%%%%%%%%%%%%%%%% COMPUTE THE ANSWER %%%%%%%%%%%%%%%%%%%
disp('Compute the answer ...... wait');
if frequency_data.AFS.flag
    doafs = 1;
else
    doafs = 0;
end

if isempty(frequency_data.AFS.max)
    doafs = 0;
elseif frequency_data.AFS.max == 0
    doafs = 0;
end

if ~doafs
    % compute the answer for exactly the frequencies that are in
    % frequency_data.frequency_points
    if m == 1
        frequency_response = zeros(nofreq,1); %SISO
    else
        frequency_response = zeros(m,m,nofreq); %MIMO
    end
    for k = 1:nofreq
        %y = compute_one_frequency(matrices,method,omega(k));
        y = solve_onelab(freqs(k),Flag_AnalysisType);
        if m == 1 % SISO
            if ~isempty(y)
                frequency_response(k) = y;
            else
                frequency_response(k) = 0;
            end
        else   % MIMO
            if ~isempty(y)
                frequency_response(:,:,k) = y;
            else
                frequency_response(:,:,k) = 0;
            end
        end
    end
else
    disp('Adaptive Frequency Sampling (AFS) is used.');
    %     disp('Adaptive Frequency Sampling (AFS) is used. Only the first and the last frequencies given are used');
    %     omega_min = omega(1);
    %     omega_max = omega(nofreq);
    %     yfmin = compute_one_frequency(matrices,method,omega_min);
    %     yfmax = compute_one_frequency(matrices,method,omega_max);
    %     listf(1) = omega_min;
    %     listf(2) = omega_max;
    %     if m == 1 % SISO
    %         listy(1) = yfmin;
    %         listy(2) = yfmax;
    %     else % MIMO
    %         listy(:,:,1) = yfmin;
    %         listy(:,:,2) = yfmax;
    %     end
    %
    listf = sort(freqs);
    for k = 1:nofreq
        %yf = compute_one_frequency(matrices,method,listf(k));
        yf = solve_onelab(freqs(k),Flag_AnalysisType);
        if m == 1 % SISO
            listy(k) = yf;
        else %MIMO
            listy(:,:,k) = full(yf);
        end
        
    end
    
    [listf,listy,trfct] = do_afs_nerecursiv(listf,listy,frequency_data.AFS,snp_info,avfitParams,Flag_AnalysisType);
                          
    nop = length(listf);
    frequency_response = listy;
    %[listf,index] = sort(listf); nu e nevoie de sortare
    if m == 1
        %frequency_response = listy(index);
        frequency_response = conj(frequency_response');
        %else
        %frequency_response = listy(:,:,index);
    end
    frequency_data.frequency_points = listf/1e9;
    frequency_data.frequency_unit = 'GHz';
    
    if nop >= frequency_data.AFS.max
        fprintf('************** WARNING **************: afs - max no of iterations = %d reached \n',...
            frequency_data.AFS.max);
        fprintf('No of computed points = %d \n',nop);
    end
    nofreq = length(listf);
end
t2 = cputime;
fprintf('Time for computing frequency response = %e \n', t2-t1);
fprintf('Number of computed frequencies = %d \n', nofreq);
disp('AFS - VF for onelab - FINISHED');
end


function [listx,listy,trfct] =  do_afs_nerecursiv(listx,listy,AFSinfo,snp_info,avfitParams,Flag_AnalysisType)
lx = length(listx);
interval_flags = ones(1,lx-1);
% 1 inseamna interval "verde", trebuie pus
% in interiorul lui un punct
no_test = nnz(interval_flags);

idx_iter = 0;
flag_ordin = 0;

mm = snp_info.nports;

while and(lx <= AFSinfo.max, no_test ~= 0)
    listx_test = zeros(1,no_test);
    flag_test = zeros(1,no_test);
    
    if mm == 1 % SISO
        listy_test = zeros(1,no_test);
    else
        listy_test = zeros(mm,mm,no_test);
    end
    
    
    idx_iter = idx_iter + 1;
    nr_pct_noi_testate(idx_iter) = no_test;
    nr_pct_testate(idx_iter) = lx + no_test;
    
    if or(strcmpi(AFSinfo.type,'vfit'),strcmpi(AFSinfo.type,'vfitlinf'))
        %%% daca se va folosi vfit, calculeaza interpolarea vfit aici
        if ~strcmpi(avfitParams.startFrom,'lowestOrder')  % starts from last order
            if idx_iter == 1
                AutoMinOrder = 1;
            else
                AutoMinOrder = ordin(idx_iter-1);
            end
            avfitParams.minOrder = AutoMinOrder;
        end
        
        
        %         AutoMaxOrder = avfitParams.maxOrder;
        %
        %         AutoNoCalls = avfitParams.noCalls;
        %         AutoTol = avfitParams.tol;
        %
        %         AutoStop = avfitParams.stopCriterion; % shows how the error is computed
        %         % it can be 0 - components for what it is (codestar way)
        %         %           1 - Frobenius S
        %         %           2 - Components S
        %
        %         Autogvfit = avfitParams.graphicsVfit; % flag to display or not grafphics of Vfit
        %         % it can be 'Yes' otherwise is treated as 'No';
        %         AutoTypeTF = avfitParams.typeTransferFunction; % type of transfer function
        %         % it can be 2 for Proper, 1 for Strictly Proper and 3 for Improper
        %         fileout = avfitParams.fileout;
        frequency_points = listx;
        freq_unit = 'Hz';
        if mm == 1 % SISO
            value = conj(listy');
        else
            value = listy;
        end
        nports = mm;
        ptype = snp_info.ptype;
        Z0 = snp_info.Z0;
        
        if idx_iter > 1
            old_poles = poles;
            old_residues = residues;
            old_kinf = kinf;
            old_prop = prop;
        end
        
        frequency_data.frequency_unit = freq_unit;
        frequency_data.frequency_points = frequency_points;
        response = value;
        
        avfitParams.idx = idx_iter;
        transferFunction =  auto_vfit3_fromfreqdata_forOnelab(...
            frequency_data,response,snp_info,avfitParams);
        
        ordin(idx_iter) = transferFunction.order;
        poles = transferFunction.poles;
        residues = transferFunction.residues;
        kinf = transferFunction.kinf;
        prop = transferFunction.prop;
        if idx_iter > 1
            if ordin(idx_iter) == ordin(idx_iter - 1)
                flag_ordin = 1;
            end
        end
    end
    
    idx = 0;
    for k = 1:lx-1 % asta va trebui distribuit
        
        if interval_flags(k) ~= 0
            x1 = listx(k);
            x2 = listx(k+1);
            if mm == 1 % SISO
                y1 = listy(k);
                y2 = listy(k+1);
            else
                y1 = listy(:,:,k);
                y2 = listy(:,:,k+1);
            end
            idx = idx + 1;
            if strcmpi(AFSinfo.type,'lin') % linear
                listx_test(idx) = (x1 + x2)/2;
                yinterp = (y1 + y2)/2;
            elseif  strcmpi(AFSinfo.type,'poly')% polytropic
                listx_test(idx) = sqrt(x1*x2);
                yinterp = sqrt(y1.*y2);
            elseif strcmpi(AFSinfo.type,'vfit')%
                % using polytropic for freq and vfit
                listx_test(idx) = sqrt(x1*x2);
                
                if mm == 1 %SISO
                    yinterp = 0;
                    omega_crt = listx_test(idx)*2*pi;
                    for k2 = 1:transferFunction.order
                        yinterp = yinterp + residues(k2)/(1i*omega_crt - poles(k2));
                    end
                    yinterp = yinterp + kinf + 1i*omega_crt*prop;
                else
                    yinterp = zeros(nop*nop,1);
                    omega_crt = listx_test(idx)*2*pi;
                    for k2 = 1:transferFunction.order
                        yinterp = yinterp + residues(:,k2)./(1i*omega_crt - poles(k2));
                    end
                    yinterp = yinterp + kinf + 1i*omega_crt*prop;
                    yinterp = reshape(yinterp,nop,nop);
                end
                
                
            elseif strcmpi(AFSinfo.type,'vfitlinf')%
                % using lin for freq and vfit
                listx_test(idx) = (x1+x2)/2;
                
                if mm == 1 %SISO
                    yinterp = 0;
                    omega_crt = listx_test(idx)*2*pi;
                    for k2 = 1:transferFunction.order
                        yinterp = yinterp + residues(k2)/(1i*omega_crt - poles(k2));
                    end
                    yinterp = yinterp + kinf + 1i*omega_crt*prop;
                else
                    yinterp = zeros(mm,mm);
                    omega_crt = listx_test(idx)*2*pi;
                    for k2 = 1:transferFunction.order
                        yinterp = yinterp + residues(:,:,k2)./(1i*omega_crt - poles(k2));
                    end
                    yinterp = yinterp + kinf + 1i*omega_crt*prop;
                    %yinterp = reshape(yinterp,mm,mm);
                end
            else
                error('unknown interpolation type');
            end
            frecvc = listx_test(idx);
            %yc = compute_one_frequency(matrices,method,omega_c); % asta va trebui paralelizat
            %omega_c = frecvc*2*pi;
            yc = solve_onelab(frecvc,Flag_AnalysisType);
            if mm == 1 % SISO
                listy_test(idx) = yc;
            else % MIMO
                listy_test(:,:,idx) = yc;
            end
            norm1 = norm(yc-yinterp,'fro');
            norm2 = norm(yc,'fro');
            %             fprintf('norm_diff = %e\n',norm1);
            %             fprintf('norm_comp = %e\n',norm2);
            %             fprintf('norm_diff/norm_comp = %e\n',norm1/norm2);
            
            if ~and(AFSinfo.err*norm2 < eps, norm1 < eps)
                if norm1 > AFSinfo.err*norm2
                    flag_test(idx) = 1;
                end
            end
            
        end
    end
    % asambleaza noua lista de frecventa si flagurile intervalelor
    lx_new = lx + no_test;
    listx_new = zeros(1,lx_new);
    if mm == 1 % SISO
        listy_new = zeros(1, lx_new);
    else
        listy_new = zeros(mm,mm,lx_new);
    end
    interval_flags_new = zeros(1,lx_new - 1);
    idx = 0;
    idx_test = 0;
    for k = 1:lx-1
        if interval_flags(k) == 1 % adauga capatul din stanga si pct intermediar
            idx_test = idx_test + 1;
            idx = idx + 1;
            listx_new(idx) = listx(k);
            if mm == 1 % SISO
                listy_new(idx) = listy(k);
            else
                listy_new(:,:,idx) = listy(:,:,k);
            end
            interval_flags_new(idx) = flag_test(idx_test); % flagul acestui noi interval depinde de flagul pct de test
            idx = idx + 1;
            listx_new(idx) = listx_test(idx_test);
            if mm == 1 % SISO
                listy_new(idx) = listy_test(idx_test);
            else
                listy_new(:,:,idx) = listy_test(:,:,idx_test);
            end
            interval_flags_new(idx) = flag_test(idx_test);
        else % pastreaza doar capatul din stanga
            idx = idx + 1;
            listx_new(idx) = listx(k);
            if mm == 1 % SISO
                listy_new(idx) = listy(k);
            else
                listy_new(:,:,idx) = listy(:,:,k);
            end
            interval_flags_new(idx) = 0; % acest interval nu a fost si nu va mai fi divizat
        end
    end
    % adauga ultimul punct
    idx = idx + 1;
    listx_new(idx) = listx(lx);
    if mm == 1 % SISO
        listy_new(idx) = listy(lx);
    else
        listy_new(:,:,idx) = listy(:,:,lx);
    end
    
    lx = lx_new;
    listx = listx_new;
    listy = listy_new;
    interval_flags = interval_flags_new;
    no_test = nnz(interval_flags);
    if flag_ordin == 1
        if no_test ~= 0
            er_poles = norm(old_poles - poles)/norm(poles);
            %er_residues = norm(old_residues - residues)/norm(residues);
            %fprintf('==================================>>>>>>>>>>>>>>>>>>er poles = %e\n',er_poles);
            %fprintf('==================================>>>>>>>>>>>>>>>>>>er residues = %e\n',er_residues);
            %if prop ~= 0
            %    er_prop = norm(old_prop - prop)/norm(prop);
            %    fprintf('==================================>>>>>>>>>>>>>>>>>>er prop = %e\n',er_prop);
            %end
            %if kinf ~= 0
            %    er_kinf = norm(old_kinf - kinf)/norm(kinf);
            %    fprintf('==================================>>>>>>>>>>>>>>>>>>er kinf = %e\n',er_kinf);
            %end
            er_tf  = snp_info.tol_poles;
            %if and(and(er_poles < er_tf,er_residues<er_tf),er_kinf<er_tf)
            %if and(er_poles < er_tf,er_residues<er_tf)
            if er_poles < er_tf
                no_test = 0; % se va opri, atentie ultimul vfit e facut pe penultimul set de puncte
                disp(' ======================================================================================');
                disp(' - STOP DUE TO THE UNCHANGE OF THE REDUCED ORDER MODEL FOR TWO CONSECUTIVE ITERATIONS -');
                disp(' - and relative difference of the poles vector less than 1 % - ');
                disp(' ======================================================================================');
            else
                flag_ordin = 0;
            end
        end
    end
end



%%%% vector fit final + sinteza
frequency_data.frequency_unit = 'Hz';
frequency_data.frequency_points = listx;
response = listy;

avfitParams.idx = idx_iter+1;
if mm == 1
    response = conj(response');
end

%avfitParams.minOrder = 1;

trfct = auto_vfit3_fromfreqdata_forOnelab(...
    frequency_data,response,snp_info,avfitParams);

if or(strcmpi(AFSinfo.type,'vfit'),strcmpi(AFSinfo.type,'vfitlinf'))
    disp('ordin');
    disp(ordin);
end
disp('nr puncte noi evaluate');
disp(nr_pct_noi_testate);
disp('nr total puncte');
disp(nr_pct_testate);

disp('Polii functiei de transfer pentru modelul final');
disp(trfct.poles);
end

