% Analytic solution of the vector Helmholtz equation in a rectangle 
% Gabriela Ciuprina
% February 4, 2020
clc;
close all hidden
% prepare path to use chamy tools
restoredefaultpath;
sourcespath = genpath('D:\Gabriela\OneLab\mytests\ECEforOnelab\ECEinOnelab_E_2021\MatlabSources\Chamy');
problempath = 'D:\Gabriela\OneLab\mytests\ECEforOnelab\ECEinOnelab_E_2021\Results_log\01_Ishape2Danalitic';
addpath(sourcespath);
addpath(problempath);
chdir(problempath);
outpath1 = 'out_extrem';
outpath2 = 'out_analitic_MQS';
outpath3 = 'out_analitic_FW';
%% geometry
d = 5e-6; %[m] thickness of the conducting plate 
geom.a = d/2; %
%zmax = 500e-6; [m]
zmax = 10e-6; %[m]
geom.l = zmax; % length of the conductor
%geom.h = 1; % the results will be per meter (in depth)
geom.h = 10e-6;
excitatie_unVolt = 0; %1; % set the flag 1 if you want a voltage excitation of 
                          % 1V, otherwise it means a current imposed to 1 A 
if ~excitatie_unVolt
    geom.crt = 1; % the analytic result is obtained easier for a current excitation
end


%% Materials
format long e;
mat.sig = 6.6e7;    % [S/m] conductivity - Aluminiu
mat.mu = 4*pi*1e-7; % [H/m] permeability - air
%% frequency range
fmin = 1e7;  %[Hz]
feddy = 3e9; %[Hz]
fmax = 100e9;%[Hz]

% frequency points
nop = 100;
freq.fvect = logspace(log10(fmin),log10(fmax),nop)';
%nop = 3;
%freq.fvect = [fmin; feddy; fmax];
%%%%
%% extreme computation - d.c. (cc = curent continuu) and strong skin depth 
if 1 == 0
    files.outpath = outpath1;
    files.problempath = problempath;
    files.rootname = '2Drectangle_xy_extrem_';
    idxfig = 1; % number of starting figure - for plots
    [R_cc_extrem,L_cc_extrem] = rectangle_analitic_extrem(geom,mat,freq,files,idxfig);
end

%% MQS computation - analytica formulas 
if 1 == 0
    files.outpath = outpath2;
    files.problempath = problempath;
    files.rootname = '2Drectangle_xy_MQS_';
    idxfig = 10; % number of starting figure - for plots
    rectangle_analitic_MQS(geom,mat,freq,files,idxfig,R_cc_extrem,L_cc_extrem);
end

%% FW computation - analytical formulas 
if 1 == 1
    mat.eps = 8.854187812812e-12; %F/m
    files.outpath = outpath3;
    files.problempath = problempath;
    files.rootname = '2Drectangle_xy_FW_';
    idxfig = 20; % number of starting figure - for plots
    rectangle_analitic_FW(geom,mat,freq,files,idxfig);
end

%% field representation in FW 
if 1 == 0
    mat.eps = 8.854187812812e-12; %F/m
    % f = fmin;
    %f = feddy;
    %f = fmax;
    f = 1e9;
    
    T = 1/f;
    tplotDivider = 4; % arrows and color maps are for a time moment  = T/tplotDivider;
    files.outpath = outpath3;
    files.problempath = problempath;
    files.rootname = '2Drectangle_xy_FW_';
    idxfig = 30; % number of starting figure - for plots
    nx = 50;
    ny = 2;
    
    %%% computation of R and L for one single frequency
    omega = 2*pi*f;
    gamma_cplx_patrat = 1i*omega*mat.mu.*(mat.sig + 1i*omega*mat.eps);
    gamma_cplx = sqrt(gamma_cplx_patrat);
    gamma_a = gamma_cplx*geom.a;
    chga = cosh(gamma_a);
    shga = sinh(gamma_a);
    
    if excitatie_unVolt == 1
        % if we want that the results corresponds to an exciation of 1 V, then the value of
        % the current has to be
        Ey_complex = 1/geom.l;
        Hz_complex = Ey_complex*(mat.sig + 1j*omega*mat.eps)/gamma_cplx*(shga./chga);
        crt = Hz_complex*2*geom.h;
    else
        crt = geom.crt;
        Ey_complex = (gamma_cplx./(mat.sig + 1j*omega*mat.eps)).*(chga./shga)*crt/(2*geom.h);
        Hz_complex = crt/(2*geom.h);
    end
    %
    
    P_ap_cplx_lineic = 2*Ey_complex*conj(Hz_complex)*geom.l*geom.h;
    Ranalitic = real(P_ap_cplx_lineic);
    X_h = imag(P_ap_cplx_lineic);
    Lanalitic = X_h./omega;
     
    fprintf('\n f = %g GHz',f/1e9);
    fprintf('\n  \t Resistance (analytic)');
    disp(Ranalitic);
    fprintf('\n \t Inductance (analytic)');
    disp(Lanalitic);
    fprintf('\n  \t Hz complex (analytic)');
    disp(Hz_complex);
    fprintf('\n  \t Ey complex (analytic)');
    disp(Ey_complex);
    fprintf('\n  \t Ey - rms (analytic)');
    disp(abs(Ey_complex));
    plot_rectangle_analitic_FW(crt,geom,mat,freq,files,idxfig,f,tplotDivider,nx,ny);
end