/* Electrokinetics_V_ece (old name "formulationEC_V_terminals.pro") 

   Electrokinetics formulation in V
   Dirichlet boundary conditions are grouped in terminals (which can be floating); 
   The non-Dirichlet (non-terminal) boundary is zero Neumann.   

   The domain is simply connected. 
   The terminals can be voltage or current excited.
   The exact seting of terminal excitaitons is defined in Ishape2d.pro (problem dependent file) 
*/

Jacobian {
  { Name Vol;
    Case {
      If(Flag_Axi && modelDim < 3)
        { Region All; Jacobian VolAxiSqu; }
      Else
        { Region All; Jacobian Vol; }
      EndIf
    }
  }
  { Name Sur;
    Case {
      If(Flag_Axi && modelDim < 3)
        { Region All; Jacobian SurAxi; }
      Else
        { Region All; Jacobian Sur; }
      EndIf
    }
  }
}

Integration {
  { Name Int;
    Case {
      { Type Gauss;
        Case {
          { GeoElement Point; NumberOfPoints  1; }
          { GeoElement Line; NumberOfPoints  3; }
          { GeoElement Triangle; NumberOfPoints  3; }
          { GeoElement Quadrangle; NumberOfPoints  4; }
          { GeoElement Tetrahedron; NumberOfPoints  4; }
          { GeoElement Hexahedron; NumberOfPoints  6; }
          { GeoElement Prism; NumberOfPoints  9; }
          { GeoElement Pyramid; NumberOfPoints  8; }
	}
      }
    }
  }
}

/* The formulation and its tools */
 Include "only_FunctionSpace_and_Formulation_Electrokinetics_V_ece.pro"

Resolution {
  { Name Electrokinetics_v;
    System {
      { Name Sys_Ele; NameOfFormulation Electrokinetics_v; }
    }
    Operation {
      Generate[Sys_Ele]; Solve[Sys_Ele]; SaveSolution[Sys_Ele];
    }
  }
}

PostProcessing {
 { Name EC_v_terminals; NameOfFormulation Electrokinetics_v;
    Quantity {
      { Name v; Value {
          Term { [ {v} ]; In Vol_EC; Jacobian Vol; }
        }
      }
      { Name E; Value {
          Term { [ -{d v} ]; In Vol_EC; Jacobian Vol; }
        }
      }      
    }
  }
}
