/* -------------------------------------------------------------------
   File "formulationsFW_E_classicBC.pro" (problem independent file)
  
   Problem independent pro file
   for passive, linear domains in Full Wave with classical boundary conditions (Et or Ht).

   Formulation with vec{E} 
   Frequency domain simulations.  
*/

Jacobian {
  { Name Vol;
    Case {
      If(Flag_Axi && modelDim < 3)
        { Region All; Jacobian VolAxiSqu; }
      Else
        { Region All; Jacobian Vol; }
      EndIf
    }
  }
  { Name Sur;
    Case {
      If(Flag_Axi && modelDim < 3)
        { Region All; Jacobian SurAxi; }
      Else
        { Region All; Jacobian Sur; }
      EndIf
    }
  }
}

Integration {
  { Name Int;
    Case {
      { Type Gauss;
        Case {
          { GeoElement Point; NumberOfPoints  1; }
          { GeoElement Line; NumberOfPoints  3; }
          { GeoElement Triangle; NumberOfPoints  3; }
          { GeoElement Quadrangle; NumberOfPoints  4; }
          { GeoElement Tetrahedron; NumberOfPoints  4; }
          { GeoElement Hexahedron; NumberOfPoints  6; }
          { GeoElement Prism; NumberOfPoints  9; }
          { GeoElement Pyramid; NumberOfPoints  8; }
	}
      }
    }
  }
}

/* The formulation and its tools */
 Include "only_FunctionSpace_and_Formulation_FullWave_E_classic.pro"

Resolution {
  {Name VoltageSource_e_BC; Hidden 1;
   System {
      { Name Bc; NameOfFormulation VoltageSource_e_BC; DestinationSystem Sys_Ele; }
    }
    Operation {
      Generate Bc; Solve Bc; TransferSolution Bc;
    }
  } 
  { Name FullWave_E_classic;
    System {
      { Name Sys_Ele; NameOfFormulation FullWave_E_classic;
        Type Complex; Frequency Freq;}
    }
    Operation {
      Generate[Sys_Ele]; Solve[Sys_Ele]; SaveSolution[Sys_Ele];
    }
  } 
}

PostProcessing {
    { Name FW_E_classic; NameOfFormulation FullWave_E_classic;
    Quantity {
      { Name E;
        Value {
          Local { [ {e} ]; In Dom_FW; Jacobian Vol; } // complex
        }
      }
     { Name B;
        Value {
          Local { [ CompZ[{d e}]/(2*Pi*Freq*Complex[0,1]) ]; In Dom_FW; Jacobian Vol; }
        }
      }
	  { Name H;
        Value {
          Local { [ nu[]*CompZ[{d e}]/(2*Pi*Freq*Complex[0,1]) ]; In Dom_FW; Jacobian Vol; }
        }
      }
    }
  }
}
