/* FullWave_E_ece_SISO.pro  (old name "formulationsFW_E_eceBC_v6.pro")  

 Problem independent pro file
 for passive, linear domains in Full Wave with ECE boundary conditions.

 It uses a formulation with vec{E} strictly inside the domain and V strictly on the boundary.
 The domain is simply connected, it has only electric terminals.
 The terminals can be voltage or current excited.
 The material inside is linear from all points of view: electric, magnetic, conduction
 It is compulsory that there exists a ground terminal (its voltage is set to zero).
 The post-operation assumes that there is one excited (active) terminal  (excitation equal to 1 - it can be in 
 voltage or current excited terminal). 
 Frequency domain simulations.
*/

Jacobian {
  { Name Vol;
    Case {
      If(Flag_Axi && modelDim < 3)
        { Region All; Jacobian VolAxi; }  //VolAxi  or VolAxiSqu ??? which one??
      Else
        { Region All; Jacobian Vol; }
      EndIf
    }
  }
  { Name Sur;
    Case {
      If(Flag_Axi && modelDim < 3)
        { Region All; Jacobian SurAxi; }
      Else
        { Region All; Jacobian Sur; }
      EndIf
    }
  }
}

Integration {
  { Name Int;
    Case {
      { Type Gauss;
        Case {
          { GeoElement Point; NumberOfPoints  1; }
          { GeoElement Line; NumberOfPoints  3; }
          { GeoElement Triangle; NumberOfPoints  3; }
          { GeoElement Quadrangle; NumberOfPoints  4; }
          { GeoElement Tetrahedron; NumberOfPoints  4; }
          { GeoElement Hexahedron; NumberOfPoints  6; }
          { GeoElement Prism; NumberOfPoints  9; }
          { GeoElement Pyramid; NumberOfPoints  8; }
	}
      }
    }
  }
}

Include "only_FunctionSpace_and_Formulation_FullWave_E_ece.pro"

Resolution {
  { Name FullWave_E_ece;
    System {
      { Name Sys_Ele; NameOfFormulation FullWave_E_ece;
        Type Complex; Frequency Freq;}

    }
    Operation {
	  SetTime[Freq];               // usefull to save (and print) the current frequency
	  //GenerateSeparate[Sys_Ele]; // separate matrices
      Generate[Sys_Ele]; Solve[Sys_Ele]; SaveSolution[Sys_Ele];
	  //Print[Sys_Ele];            // save in matlab format or change the format when running getdp 

    }
  }

}

PostProcessing {

  { Name FW_E_ece; NameOfFormulation FullWave_E_ece;
    Quantity {
      { Name E;
        Value {
          Local { [ {e} ]; In Vol_FW; Jacobian Vol; } // complex
        }
      }

      { Name J;
        Value {
          Local { [ sigma[]*{e} ]; In Vol_FW; Jacobian Vol; } // complex
        }
      }

      { Name rmsJ;
        Value {
          Local { [ Norm[sigma[]*{e}] ]; In Vol_FW; Jacobian Vol; } // real
        }
      }

      { Name gradV;
        Value {
          Local { [ {dv} ]; In Vol_FW; Jacobian Vol; } // complex
        }
      }

      { Name Vterminals;
        Value {
          // This is a global variable, you do not need to indicate the Jacobian
          // It is only defined at the terminal and it is only one value for the whole line (surface in 3D)
          // If the support you give here is wider, you will get 0 where it is not defined
		  Term { [ -1*{V} ]; In Sur_Terminals_FWece; } // => unknowns only in Sur_Terminals_FWece, outside it will be zero
        }
      }

      { Name I; 
		Value { 
			Term { [ -1*{I}*h2Ddepth ]; In Sur_Terminals_FWece; } 
		}
	  }  // h2Ddepth is the depth for 2D problems,  and it has to be 1 for 3D problems

      { Name VnotTerminals;
        Value {
          Local { [ -{dInv dv} ]; In Vol_FW; Jacobian Vol; }
        }
      }

       { Name B;
         Value {
           Local { [ CompZ[{d e}]/(2*Pi*Freq*Complex[0,1]) ]; In Vol_FW; Jacobian Vol; }
         }
       }

       { Name H;
         Value {
           Local { [ nu[]*CompZ[{d e}]/(2*Pi*Freq*Complex[0,1]) ]; In Vol_FW; Jacobian Vol; }
         }
       }
	   
	   //{ Name Dummy;
       //  Value {
       //    //Local { [ 1 ]; In MaterialSi; Jacobian Vol; }
	   //   Local { [ 1 ]; In MaterialSiO2; Jacobian Vol; }
       // }
       //}
	   
     }
   }
 }
