function [ptype,pformatfile,frequency_points,value,Z0, freq_unit,nports] = loadsnp(filename)

%loads data from a snp
%
% syntax :   [ptype,pformatfile,freq_point,value,Z0] = loadsnp(filename)
%               ptype        - type of parameter in file (Y,Z,S)
%               pformatfile      - type of representation used (MA,DB or RI)
%               freq_point   - vector of frequency points
%               value        - the value of the parameter in freq_points
%               Z0           - normalizing impedance for S parameter
%

ll = length(filename);
if strcmp(filename(ll-3:ll),'.mat')
    load(filename); % aici sunt variabilele
    % 'ptype','pformat','freq_points','data','Z0','units','nports','-mat');
    pformatfile = pformat;
    frequency_points = freq_points;
    value = data;
    freq_unit = units;
else

    frecv_unit=[];       %frequency unit
    parameter=[];   %parameter type G H S Y Z
    formatfile=[];      %formatfile type MA DB RI
    frequency_points = [];
    data = [];
    format long e;

    fid=fopen(filename);
    if fid == -1
        error(sprintf('==========> loadsnp.m: file %s cannot be opened',filename));
    end
    img = sqrt(-1); % just to be sure

    % -----------------------------------------------------
    % get the header
    % -----------------------------------------------------

    s=fgetl(fid);
    %disp(s);
    while ischar(s)
        %disp(s)
        if isempty(s)
            s=fgetl(fid);
        else
            if or (strcmp(s(1),'!'),strcmp(s(1),' '))
                s=fgetl(fid);
            elseif strcmp(s(1),'#')
                break;
            end
        end
    end

    k=2;
    while isspace(s(k))  %s(k) == ' '
        k=k+1;
    end;
    while ~isspace(s(k))  %s(k) ~= ' '
        frecv_unit=[frecv_unit s(k)];
        k=k+1;
    end
    while isspace(s(k))  %s(k) == ' '
        k=k+1;
    end;
    while ~isspace(s(k))  %s(k) ~= ' '
        parameter=s(k);
        k=k+1;
    end
    while isspace(s(k))  %s(k) == ' '
        k=k+1;
    end;
    while ~isspace(s(k))  %s(k) ~= ' '
        formatfile=[formatfile s(k)];
        k=k+1;
    end
    while isspace(s(k))  %s(k) == ' '
        k=k+1;
    end;
    k=k+1;
    while isspace(s(k))  %s(k) == ' '
        k=k+1;
    end;
    Z0char=s(k:length(s));
    Z0=str2double(Z0char);
    ptype=parameter;
    pformatfile=upper(formatfile);

    lenf = length(filename);
    fin = lenf-1;
    while ~strcmp(filename(fin-1),'s')
        fin = fin - 1;
    end

    nports = filename(fin:lenf-1);
    nports = str2double(nports);

    nocol = nports*nports*2;
    formatstring = ' %e';
    for i = 1:nocol
        formatstring = strcat(formatstring,' %e');
    end
    nr = 0;
    while ~feof(fid)
        try
            M = fscanf(fid,formatstring,nocol+1);
            frequency_points = [frequency_points; M(1)];
            nr = nr+1;
            for k = 1:nocol
                data(nr,k)= M(k+1);
            end
        catch
            fgetl(fid);
        end
    end;
    if nports == 1 % SISO
        switch pformatfile
            case 'RI'
                value=data(:,1) + img*data(:,2);
            case 'MA'
                %realpart = data(:,1).*cos(data(:,2));
                %imagpart = data(:,1).*sin(data(:,2));
                realpart = data(:,1).*cos(data(:,2)*pi/180);
                imagpart = data(:,1).*sin(data(:,2)*pi/180);
                value=realpart + img*imagpart;
            case 'DB'
                modulpart = 10.^(data(:,1)/20);
                %realpart = modulpart.*cos(data(:,2));
                %imagpart = modulpart.*sin(data(:,2));
                realpart = modulpart.*cos(data(:,2)*pi/180);
                imagpart = modulpart.*sin(data(:,2)*pi/180);
                value=realpart + img*imagpart;
            otherwise
                disp('loadsnp: unknown format for snp file. Only RI, MA, DB are accepted');
                return;
        end
    else  % MIMO
        index = 1;
        switch pformatfile
            case 'RI'
                for j = 1:nports
                    for i = 1:nports
                        value(i,j,:) = data(:,index)+ img*data(:,index+1);
                        index = index + 2;
                    end
                end
            case 'MA'
                for j = 1:nports
                    for i = 1:nports
                        %realpart = data(:,index).*cos(data(:,index+1));
                        %imagpart = data(:,index).*sin(data(:,index+1));
                        realpart = data(:,index).*cos(data(:,index+1)*pi/180);
                        imagpart = data(:,index).*sin(data(:,index+1)*pi/180);
                        value(i,j,:) = realpart + img*imagpart;
                        index = index + 2;
                    end
                end
            case 'DB'
                for j = 1:nports
                    for i = 1:nports
                        modulpart = 10.^(data(:,index)/20);
                        %realpart = modulpart.*cos(data(:,index+1));
                        %imagpart = modulpart.*sin(data(:,index+1));
                        realpart = modulpart.*cos(data(:,index+1)*pi/180);
                        imagpart = modulpart.*sin(data(:,index+1)*pi/180);
                        value(i,j,:)=realpart + img*imagpart;
                        index = index + 2;
                    end
                end
            otherwise
                disp('loadsnp: unknown format for snp file. Only RI, MA, DB are accepted');
                return;
        end

    end
    freq_unit = frecv_unit;
    fclose(fid);
end
