function frequency_data = loadsnpfreq_v2(filename)

% loads frequencies from a snp file
%
% syntax :   frequency_data = loadsnp(filename)
%          frequency_data is a structure with the following fields
%               ptype        - type of parameter in file (Y,Z,S,G,H)
%               pformatfile  - type of representation used (MA,DB or RI)
%               freq_point   - vector of frequency points
%               Z0           - normalizing impedance for S parameter               
%
fid = fopen(filename);
if fid == -1
    error('===========================> ERROR OPENING %s. PLEASE CHECK THAT THIS FILE EXISTS',filename);
end

s = fgetl(fid); % get the headear
while ischar(s)
    if or (strcmp(s(1),'!'),strcmp(s(1),' '))
        s = fgetl(fid);
    elseif strcmp(s(1),'#')
        break;
    else
        disp('strange first line in the snp file');
        disp('the line is');
        disp(s);
        error('check the snp file again.');
    end
end

k=2;
while s(k) == ' '
    k = k+1;
end;
frequency_unit = [];
while s(k) ~= ' '
    frequency_unit = [frequency_unit s(k)];
    k = k+1;
end
while s(k) == ' '
    k = k+1;
end;
parameter = [];
while s(k) ~= ' '
    parameter = [parameter s(k)];
    k = k+1;
end
while s(k) == ' '
    k = k+1;
end;
formatfile = [];
while s(k) ~= ' '
    formatfile = [formatfile s(k)];
    k = k+1;
end
while s(k) == ' '
    k=k+1;
end;
k = k + 1;
while s(k) == ' '
    k = k + 1;
end;
Z0char = s(k:length(s));
Z0 = str2double(Z0char);
ptype = parameter;
pformatfile = upper(formatfile);

frequency_points = [];
while ~feof(fid)
    try
        linie = fgetl(fid);
        linie = str2num(linie);
        if length(linie) == 1
            frequency_points = [frequency_points; linie];
        else
            val = linie(1);
            frequency_points = [frequency_points; val];
        end
    catch
        s = fgetl(fid);
    end
end;

disp(sprintf('File: %s => Number of frequencies = %d',filename,length(frequency_points)));

fclose(fid);
frequency_data.ptype = ptype; % type of parameter in file (Y,Z,S)
frequency_data.pformatfile = pformatfile; % type of representation used (MA,DB or RI)
frequency_data.frequency_points = frequency_points;
frequency_data.frequency_unit = frequency_unit;
frequency_data.Z0 = Z0; % normalizing impedance for S parameter    


