function [ok] = plot_snp(ptypes,pformatfiles,frequency_points,...
    freq_unit,dataall,tit,nooffiles,Z0,tipx,tipy,posleg,errortype)

ok = 0;
[n,m,ll] = size(dataall{1});
%disp(sprintf('Systems with %d inputs/outputs', m));
units = freq_unit{1};

ptype = upper(ptypes{1});
pformatfile = upper(pformatfiles{1});
linew = 2;

switch pformatfile
    case 'RI'
        ylabel1 = 'Real part';
        ylabel2 = 'Imaginary part';
    case 'MA'
        ylabel1 = 'Magnitude (absolute)';
        ylabel2 = 'Phase (degree)';
    case 'DB'
        ylabel1 = 'Magnitude (decibels)';
        ylabel2 = 'Phase (degree)';
    otherwise
        disp('plot_snp error: Unknows format. Only RI, MA, DB implemented');
        ok = 1;
        return;
end
x_label = strcat('Frequency [',freq_unit,']');
y_label1 = strcat(ylabel1,'---',ptype);
y_label2 = strcat(ylabel2,'---',ptype);
img = sqrt(-1);
colors =    {'-b','--r*','m:' ,'g','y','k','c','r','g','y','k','c','b','r','m:' ,'g','y'};
%markers =   ['-','*' ,'+' ,'d','+','h','p','+','*' ,'o' ,'d','+','h','p','*' ,'o' ,'d'];


for i = 1:nooffiles % replace _ with - in filename strings
    tittmp = tit{i};
    zz = findstr('_',tittmp);
    tittmp(zz) = '-';
    tit{i} = tittmp;
end
if nooffiles == 2
    tit{3} = 'absolute error';
end

if nooffiles == 1   % only 1 file - for characterize
    freq_points = frequency_points{1};
    data = squeeze(dataall{1});
    if m == 1 % SISO
        realpart = real(data);
        imagpart = imag(data);
        switch pformatfile
            case 'RI'
                first = realpart;
                second = imagpart;
            case 'MA'
                first = abs(data);
                second = angle(data)*180/pi;
            case 'DB'
                first = 20*log10(abs(data));
                second = angle(data)*180/pi;
        end
        if and(strcmp(tipx,'Xaxis-log'),strcmp(tipy,'Yaxis-log'))
            figure(1); loglog(freq_points,first,colors{1},'LineWidth',linew);
            figure(2); loglog(freq_points,second,colors{1},'LineWidth',linew);
        elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-log'))
            figure(1); semilogy(freq_points,first,colors{1},'LineWidth',linew);
            figure(2); semilogy(freq_points,second,colors{1},'LineWidth',linew);
        elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-lin'))
            figure(1); plot(freq_points,first,colors{1},'LineWidth',linew);
            figure(2); plot(freq_points,second,colors{1},'LineWidth',linew);
        else 
            figure(1); semilogx(freq_points,first,colors{1},'LineWidth',linew);
            figure(2); semilogx(freq_points,second,colors{1},'LineWidth',linew);
        end
        figure(1); grid on; xlabel(x_label(1)); ylabel(y_label1); legend(tit{1});
        figure(2); grid on; xlabel(x_label(1)); ylabel(y_label2); legend(tit{1});
    else      % MIMO
        index = 1;
        for i = 1:m
            for j = 1:m
                realpart = squeeze(real(data(i,j,:)));
                imagpart = squeeze(imag(data(i,j,:)));
                switch pformatfile
                    case 'RI'
                        first = realpart;
                        second = imagpart;
                    case 'MA'
                        datac = realpart+img*imagpart;
                        first = abs(datac);
                        second = angle(datac)*180/pi;
                    case 'DB'
                        datac = realpart+img*imagpart;
                        first = 20*log10(abs(datac));
                        second = angle(datac)*180/pi;
                end
                % plot
                if and(strcmp(tipx,'Xaxis-log'),strcmp(tipy,'Yaxis-log'))
                    figure(index); loglog(freq_points,first,colors{1},'LineWidth',linew);
                    figure(index+1); loglog(freq_points,second,colors{1},'LineWidth',linew);
                elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-log'))
                    figure(index); semilogy(freq_points,first,colors{1},'LineWidth',linew);
                    figure(index+1); semilogy(freq_points,second,colors{1},'LineWidth',linew);
                elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-lin'))
                    figure(index); plot(freq_points,first,colors{1},'LineWidth',linew);
                    figure(index+1); plot(freq_points,second,colors{1},'LineWidth',linew);
                else  % 
                    figure(index); semilogx(freq_points,first,colors{1},'LineWidth',linew);
                    figure(index+1); semilogx(freq_points,second,colors{1},'LineWidth',linew);
                end
                y_lab1 = strcat(y_label1,num2str(i),num2str(j));
                y_lab2 = strcat(y_label2,num2str(i),num2str(j));
                figure(index); grid on; xlabel(x_label); ylabel(y_lab1); legend(tit{1});
                figure(index+1); grid on; xlabel(x_label); ylabel(y_lab2); legend(tit{1});
                index = index + 2;
            end
        end
    end
else   % multiple files - compare and compute errors
    % check dimension; the frequency units are assumed to be the same
    for i = 2:nooffiles
        [n,m2,nm] = size(dataall{i});
        if m2 ~= m
            disp('Error: Models do not have the same no of outputs. You cannot compare them');
            ok = 1;
            return;
        end
        if ~strcmp(ptypes{i},ptype)
            fprintf('Model %s  type is = %s \n',tit{i},ptypes{i});
            fprintf('Model %s  type is = %s \n',tit{1},ptypes{1});
            disp('Error: you cannot compare these files. Convert them.');
            ok = 1;
            return;
        end
        if ~strcmp(pformatfiles{i},pformatfile)
            fprintf('Model %s format is = %s \n',tit{i},pformatfiles{i});
            fprintf('Model %s format is = %s \n',tit{1},pformatfiles{1});
            disp('Error: you cannot compare these files. Convert them.');
            ok = 1;
            return;
        end
        
        if ~strcmpi(freq_unit{i},units)
            fprintf('Model %s freq unit is = %s \n',tit{i},freq_unit{i});
            fprintf('Model %s freq unit is = %s \n',tit{1},freq_unit{1});
            disp('Error: you cannot compare these files. Convert them.');
            ok = 1;
            return;
        end
        
    end  % end check
    % begin plot
    if m == 1 % SISO
        for i = 1:nooffiles
            freq_points = frequency_points{i};
            data = squeeze(dataall{i});
            realpart = real(data);
            imagpart = imag(data);
            switch pformatfile
                case 'RI'
                    first = realpart;
                    second = imagpart;
                case 'MA'
                    first = abs(data);
                    second = angle(data)*180/pi;
                case 'DB'
                    first = 20*log10(abs(data));
                    second = angle(data)*180/pi;
            end
            if i == 1
                first_ref = first;
                second_ref = second;
                freq_points_ref = freq_points;
            end
            if i == 2
                
                first_interp = interp1(freq_points,first,freq_points_ref,'linear','extrap');
                second_interp = interp1(freq_points,second,freq_points_ref,'linear','extrap');
            end
            if and(strcmp(tipx,'Xaxis-log'),strcmp(tipy,'Yaxis-log'))
                figure(1); if i~= 1, hold on; end; loglog(freq_points,first,colors{i},'LineWidth',linew);
                figure(2); if i~= 1, hold on; end; loglog(freq_points,second,colors{i},'LineWidth',linew);
                if i == 2
                    if nooffiles == 2
                        figure(1); hold on; loglog(freq_points_ref,abs(first_interp-first_ref),colors{i+1},'LineWidth',linew);
                        figure(2); hold on; loglog(freq_points_ref,abs(second_interp-second_ref),colors{i+1},'LineWidth',linew);
                    end
                end
            elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-log'))
                figure(1); if i~= 1, hold on; end; semilogy(freq_points,first,colors{i},'LineWidth',linew);
                figure(2); if i~= 1, hold on; end; semilogy(freq_points,second,colors{i},'LineWidth',linew);
                if i == 2
                    if nooffiles == 2
                        figure(1); hold on; semilogy(freq_points_ref,abs(first_interp-first_ref),colors{i+1},'LineWidth',linew);
                        figure(2); hold on; semilogy(freq_points_ref,abs(second_interp-second_ref),colors{i+1},'LineWidth',linew);
                    end
                end
            elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-lin'))
                figure(1); if i~= 1, hold on; end; plot(freq_points,first,colors{i},'LineWidth',linew);
                figure(2); if i~= 1, hold on; end; plot(freq_points,second,colors{i},'LineWidth',linew);
                if i == 2
                    if nooffiles == 2
                        figure(1); hold on; plot(freq_points_ref,abs(first_interp-first_ref),colors{i+1},'LineWidth',linew);
                        figure(2); hold on; plot(freq_points_ref,abs(second_interp-second_ref),colors{i+1},'LineWidth',linew);
                    end
                end
            else  % this is default: log on x and lin on
                figure(1); if i~= 1, hold on; end; semilogx(freq_points,first,colors{i},'LineWidth',linew);
                figure(2); if i~= 1, hold on; end; semilogx(freq_points,second,colors{i},'LineWidth',linew);
                if i == 2
                    if nooffiles == 2
                        figure(1); hold on; semilogx(freq_points_ref,abs(first_interp-first_ref),colors{i+1},'LineWidth',linew);
                        figure(2); hold on; semilogx(freq_points_ref,abs(second_interp-second_ref),colors{i+1},'LineWidth',linew);
                    end
                end
            end
        end
        figure(1); grid on; xlabel(x_label(1)); ylabel(y_label1); legend(tit);
        figure(2); grid on; xlabel(x_label(1)); ylabel(y_label2); legend(tit);
    else      % MIMO
        for ii = 1:nooffiles
            freq_points = frequency_points{ii};
            data = dataall{ii};
            index = 1;
            for i = 1:m
                for j = 1:m
                    realpart = squeeze(real(data(i,j,:)));
                    imagpart = squeeze(imag(data(i,j,:)));
                    switch pformatfile
                        case 'RI'
                            first = realpart;
                            second = imagpart;
                        case 'MA'
                            datac = realpart+img*imagpart;
                            first = abs(datac);
                            second = angle(datac)*180/pi;
                        case 'DB'
                            datac = realpart+img*imagpart;
                            first = 20*log10(abs(datac));
                            second = angle(datac)*180/pi;
                    end
                    if ii == 1
                        first_ref{i,j} = first;
                        second_ref{i,j} = second;
                        freq_points_ref{i,j} = freq_points;
                    end
                    if ii == 2
                        first_interp = interp1(freq_points,first,freq_points_ref{i,j},'linear','extrap');
                        second_interp = interp1(freq_points,second,freq_points_ref{i,j},'linear','extrap');
                    end
                    % plot
                    if and(strcmp(tipx,'Xaxis-log'),strcmp(tipy,'Yaxis-log'))
                        figure(index); if ii~= 1, hold on; end; loglog(freq_points,first,colors{ii},'LineWidth',linew);
                        figure(index+1); if ii~= 1, hold on; end; loglog(freq_points,second,colors{ii},'LineWidth',linew);
                        if ii == 2
                            if nooffiles == 2
                                figure(index); hold on; loglog(freq_points_ref{i,j},abs(first_interp-first_ref{i,j}),colors{ii+1},'LineWidth',linew);
                                figure(index+1); hold on; loglog(freq_points_ref{i,j},abs(second_interp-second_ref{i,j}),colors{ii+1},'LineWidth',linew);
                            end
                        end
                    elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-log'))
                        figure(index); if ii~= 1, hold on; end; semilogy(freq_points,first,colors{ii},'LineWidth',linew);
                        figure(index+1); if ii~= 1, hold on; end; semilogy(freq_points,second,colors{ii},'LineWidth',linew);
                        if ii == 2
                            if nooffiles == 2
                                figure(index); hold on; semilogy(freq_points_ref{i,j},abs(first_interp-first_ref{i,j}),colors{ii+1},'LineWidth',linew);
                                figure(index+1); hold on; semilogy(freq_points_ref{i,j},abs(second_interp-second_ref{i,j}),colors{ii+1},'LineWidth',linew);
                            end
                        end
                    elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-lin'))
                        figure(index); if ii~= 1, hold on; end; plot(freq_points,first,colors{ii},'LineWidth',linew);
                        figure(index+1); if ii~= 1, hold on; end; plot(freq_points,second,colors{ii},'LineWidth',linew);
                        if ii == 2
                            if nooffiles == 2
                                figure(index); hold on; plot(freq_points_ref{i,j},abs(first_interp-first_ref{i,j}),colors{ii+1},'LineWidth',linew);
                                figure(index+1); hold on; plot(freq_points_ref{i,j},abs(second_interp-second_ref{i,j}),colors{ii+1},'LineWidth',linew);
                            end
                        end
                    else  % this is default: log on x and lin on
                        figure(index); if ii~= 1, hold on; end; semilogx(freq_points,first,colors{ii},'LineWidth',linew);
                        figure(index+1); if ii~= 1, hold on; end; semilogx(freq_points,second,colors{ii},'LineWidth',linew);
                        if ii == 2
                            if nooffiles == 2
                                figure(index); hold on; semilogx(freq_points_ref{i,j},abs(first_interp-first_ref{i,j}),colors{ii+1},'LineWidth',linew);
                                figure(index+1); hold on; semilogx(freq_points_ref{i,j},abs(second_interp-second_ref{i,j}),colors{ii+1},'LineWidth',linew);
                            end
                        end
                        
                    end
                    y_lab1 = strcat(y_label1,num2str(i),num2str(j));
                    y_lab2 = strcat(y_label2,num2str(i),num2str(j));
                    figure(index); grid on; xlabel(x_label(1)); ylabel(y_lab1);
                    figure(index+1); grid on; xlabel(x_label(1)); ylabel(y_lab2);
                    index = index + 2;
                end
            end
        end
        for i = 1:index-1, figure(i), legend(tit); end
        % compute errors
    end
    if ~strcmp(errortype,'None')
        dataref = dataall{1};
        dataan = dataall{2};
        
        if m ~= 1 % MIMO
            for i = 1:m
                for j = 1:m
                    %ref = dataref(i,j,:);
                    an = dataan(i,j,:);
                    %aninterp = interp1q(frequency_points{2},squeeze(an),frequency_points{1});
                    aninterp = interp1(frequency_points{2},squeeze(an),frequency_points{1},'linear','extrap');
                    anint(i,j,:) = aninterp;
                end
            end
        else %SISO
            %ref = dataref;
            an = dataan;
            anint = interp1(frequency_points{2},an,frequency_points{1},'linear','extrap');
        end
        
        switch errortype
            case 'Frobenius'
                disp('error = Frobenius');
                disp('--> computed as');
                disp(' rms||X - Xref||_F / max ||Xref||_F||  ');
                fprintf(' where X is either Z or Y, depending on the initial snp file, in this case %s \n',ptype);
                stop = 0;
            case 'Frobenius S'
                disp('error = Frobenius S');
                disp('--> error used is');
                disp(' rms||S - Sref||_F / max ||Sref||_F||  ');
                stop = 1;
                
            case 'Components S'
                disp('error = Components S');
                disp('--> error used  is');
                disp(' max rms(S_i,j - Sref_i,j)');
                stop = 2;
        end
        er = compute_error(m,ll,ptypes,dataref,anint,Z0{1},Z0{2},stop);
    end
end

% Auxiliary part, where characteristics 11 and 21 are merged in one graphic
if nooffiles == 1   % only 1 file - for characterize
    if m~=1      % MIMO
        index = 1;
        for i = 1:m
            for j = 1:m
                realpart = squeeze(real(data(i,j,:)));
                imagpart = squeeze(imag(data(i,j,:)));
                switch pformatfile
                    case 'RI'
                        first = realpart;
                        second = imagpart;
                    case 'MA'
                        datac = realpart+img*imagpart;
                        first = abs(datac);
                        second = angle(datac)*180/pi;
                    case 'DB'
                        datac = realpart+img*imagpart;
                        first = 20*log10(abs(datac));
                        second = angle(datac)*180/pi;
                end
                % plot
                figure(9)
                hold on
                if i==1&j==1
                    if and(strcmp(tipx,'Xaxis-log'),strcmp(tipy,'Yaxis-log'))
                        loglog(freq_points,first,colors{2},'LineWidth',linew);
                    elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-log'))
                        semilogy(freq_points,first,colors{2},'LineWidth',linew);
                    elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-lin'))
                        plot(freq_points,first,colors{2},'LineWidth',linew);
                    else 
                        semilogx(freq_points,first,colors{2},'LineWidth',linew);
                    end
                    y_label1 = strcat(y_label1,num2str(i),num2str(j));
                    leg1 = strcat(ptype,num2str(i),num2str(j));
                end
                if i==2&j==1
                    if and(strcmp(tipx,'Xaxis-log'),strcmp(tipy,'Yaxis-log'))
                        loglog(freq_points,first,colors{1},'LineWidth',linew);
                    elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-log'))
                        semilogy(freq_points,first,colors{1},'LineWidth',linew);
                    elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-lin'))
                        plot(freq_points,first,colors{1},'LineWidth',linew);
                    else  
                        semilogx(freq_points,first,colors{1},'LineWidth',linew);
                    end
                    y_label2 = strcat(ptype,num2str(i),num2str(j));
                end
            end
            y_lab = [y_label1,', ',y_label2];
            figure(9); grid on; xlabel(x_label); ylabel(y_lab);
            %legend([leg1,' ',tit{1}], [y_label2,' ',tit{1}],posleg);
            hold off
        end
    else    % SISO
        realpart = squeeze(real(data));
        imagpart = squeeze(imag(data));
        switch pformatfile
            case 'RI'
                first = realpart;
                second = imagpart;
            case 'MA'
                datac = realpart+img*imagpart;
                first = abs(datac);
                second = angle(datac)*180/pi;
            case 'DB'
                datac = realpart+img*imagpart;
                first = 20*log10(abs(datac));
                second = angle(datac)*180/pi;
        end
        % plot
        figure(9)
        hold on
        if and(strcmp(tipx,'Xaxis-log'),strcmp(tipy,'Yaxis-log'))
            loglog(freq_points,first,colors{2},'LineWidth',linew);
        elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-log'))
            semilogy(freq_points,first,colors{2},'LineWidth',linew);
        elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-lin'))
            plot(freq_points,first,colors{2},'LineWidth',linew);
        else  % this is default: log on x and lin on
            semilogx(freq_points,first,colors{2},'LineWidth',linew);
        end
        leg1 = strcat(ptype,': first component');
        if and(strcmp(tipx,'Xaxis-log'),strcmp(tipy,'Yaxis-log'))
            loglog(freq_points,second,colors{1},'LineWidth',linew);
        elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-log'))
            semilogy(freq_points,second,colors{1},'LineWidth',linew);
        elseif and(strcmp(tipx,'Xaxis-lin'),strcmp(tipy,'Yaxis-lin'))
            plot(freq_points,second,colors{1},'LineWidth',linew);
        else  % this is default: log on x and lin on
            semilogx(freq_points,second,colors{1},'LineWidth',linew);
        end
        y_label2 = strcat(ptype,': second component');
    end
    y_lab = [y_label1,', ',y_label2];
    figure(9); grid on; xlabel(x_label); ylabel(y_lab);
    legend([leg1,' ',tit{1}], [y_label2,' ',tit{1}]);
    hold off
    
    
end