function snp_minus_snp_v2(snp_pathname1,snp_filename1,...
    snp_pathname2,snp_filename2,new_filename)

[ptype1,pformatfile1,frequency_points1,value1,Z0,freq_unit1,nports1] = ...
    loadsnp(strcat(snp_pathname1,snp_filename1));
[ptype2,pformatfile2,frequency_points2,value2,Z0,freq_unit2,nports2] = ...
    loadsnp(strcat(snp_pathname2,snp_filename2));

extension = strcat('.s',num2str(nports1),'p');
lext = length(extension);
if isempty(new_filename)
    ll = length(snp_filename1);
    ll2 = length(snp_filename2);
    new_filename = strcat(snp_filename1(1:ll-lext),'_minus_',...
        snp_filename2(1:ll2-lext),extension);
else
    ll = length(new_filename);
    if ll > lext
        if ~strcmp(new_filename(ll-lext+1:ll),extension)
            % wrong extension - add correct extension
            new_filename = strcat(new_filename,extension);
        end
    else
        new_filename = strcat(new_filename,extension);
    end
end

if nports1 ~= nports2
    error('snp_minus_snp_v2: can be used only if both models have the same no of ports');
end

if ~strcmp(ptype1,ptype2)
    error('snp_minus_snp_v2: can be used only if both models are of the same type:Z, Y or S');
end

if ~strcmp(freq_unit1,freq_unit2)
    error('snp_minus_snp_v2: can be used only if both models have the frequency expressed in the same units');
end

lf1 = length(frequency_points1);
lf2 = length(frequency_points2);
if lf1 == lf2
    if norm(frequency_points1 - frequency_points2) < eps
        valuedif = value1 - value2;
        writesnp_v2(new_filename, frequency_points1, valuedif, ptype1, freq_unit1,Z0,pformatfile1);
        disp(sprintf('--new file written: %s',new_filename));
    else
        interp_and_subtract(nports1,lf1,lf2,frequency_points1,frequency_points2,value1,value2,...
            ptype1, freq_unit1,Z0,pformatfile1,new_filename);
    end
else
    interp_and_subtract(nports1,lf1,lf2,frequency_points1,frequency_points2,value1,value2,...
        ptype1, freq_unit1,Z0,pformatfile1,new_filename);
end
end

function interp_and_subtract(nports,lf1,lf2,frequency_points1,frequency_points2,...
    value1,value2,ptype1, freq_unit1,Z0,pformatfile1,new_filename)
% if lf1 > lf2
%     new_freq = frequency_points1;
%     for i = 1:lf2
%         [new_freq] = add_real_to_vector(frequency_points2(i),new_freq);
%     end
% else
%     new_freq = frequency_points2;
%     for i = 1:lf1
%         [new_freq] = add_real_to_vector(frequency_points1(i),new_freq);
%     end
% end
% value1_new = interp1(frequency_points1,value1,new_freq,'linear','extrap');
% value2_new = interp1(frequency_points2,value2,new_freq,'linear','extrap');
% valuedif = value1_new - value2_new;
% writesnp_new(new_filename, new_freq, valuedif, ptype1, freq_unit1,Z0,pformatfile1);
if nports == 1
    value2_new = interp1(frequency_points2,value2,frequency_points1,'linear','extrap');
else
    for i = 1:nports
        for j = 1:nports
            tointerp = squeeze(value2(i,j,:));
            compval2interp = interp1(frequency_points2,tointerp,frequency_points1,'linear','extrap');
            value2_new(i,j,:) = compval2interp;
        end
    end
end
valuedif = value1 - value2_new;
%writesnp_new(new_filename, frequency_points1, valuedif, ptype1,freq_unit1,Z0,pformatfile1);
writesnp_v2(new_filename, frequency_points1, valuedif, ptype1, ...
    freq_unit1,Z0,pformatfile1);
disp(sprintf('--new file written: %s',new_filename));
end
