function [ok] = writesnp_v2(filename, freq_points, data, ptype, units,Z0,pformat)

% Saves to a snp file

% ----------------------------------------------------
% Writing header
% ----------------------------------------------------

[n,m,nm] = size(data);
if strcmpi(units,'HZ')
    freq_points = freq_points./1e9;
elseif strcmpi(units,'KHZ')
    freq_points = freq_points./1e6;
elseif strcmpi(units,'MHZ')
    freq_points = freq_points./1e3;
elseif strcmpi(units,'GHZ')
    %disp('Freq is in GHz');
else%  if strcmpi(units,'RAD/S')
    freq_points = freq_points./2/pi/1e9;
end;

units = 'GHz';

if m > 6
    nports = m;
    newname = strcat(filename,'.mat');
    pformat = 'RI';
    save(newname,'ptype','pformat','freq_points','data','Z0','units','nports','-mat');
    fprintf('!!!!!!!!! READ THIS !!!!!!!!! Output written in %s \n',newname);
    ok = 0;
else
    fid = fopen(filename,'w');
    if fid == -1
        error('Error in opening file %s',filename);
    end
    fprintf(fid,'%s %s %s %s %s %s','#',units,ptype,pformat,'R',num2str(Z0));

    dim = length(freq_points);
    ok=0;
    img = sqrt(-1); % just to be sure


    if m == 1
        A(:,1) = freq_points;
        data = squeeze(data);
        realpart = real(data);
        imagpart = imag(data);
        switch upper(pformat)
            case 'RI'
                A(:,2) = realpart;
                A(:,3) = imagpart;
            case 'MA'
                datac = realpart+img*imagpart;
                A(:,2) = abs(datac);
                A(:,3) = angle(datac)*180/pi;
            case 'DB'
                datac = realpart+img*imagpart;
                A(:,2) = 20*log10(abs(datac));
                A(:,3) = angle(datac)*180/pi;
        end
        for index = 1 : dim
            fprintf(fid, '\n%1.16e %1.16e %1.16e',A(index,1),A(index,2),A(index,3));
        end;
        %fprintf(fid, '\n%s','! end of data');
        fclose(fid);
        fprintf('Output written in %s \n',filename);
        fprintf('Current directory is %s \n',pwd);
    else % MIMO
        noval = m*m*2;
        A = zeros(nm,noval+1);
        A(:,1) = freq_points;
        index = 1;
        for j = 1:m
            for i = 1:m
                realpart = squeeze(real(data(i,j,:)));
                imagpart = squeeze(imag(data(i,j,:)));
                switch pformat
                    case 'RI'
                        index = index+1;
                        A(:,index) = realpart;
                        index = index+1;
                        A(:,index) = imagpart;
                    case 'MA'
                        datac = realpart+img*imagpart;
                        index = index+1;
                        A(:,index) = abs(datac);
                        index = index+1;
                        A(:,index) = angle(datac)*180/pi;
                    case 'DB'
                        datac = realpart+img*imagpart;
                        index = index+1;
                        A(:,index) = 20*log10(abs(datac));
                        index = index + 1;
                        A(:,index) = angle(datac)*180/pi;
                end
            end
        end

        for k = 1 : dim
            fprintf(fid, '\n');
            fprintf(fid, '%1.16e ',A(k,1));
            %fprintf(fid, '%2.20g ',A(k,1));
            for i = 2:noval+1
                fprintf(fid, ' %1.16e ',A(k,i));
                %fprintf(fid, ' %2.20g ',A(k,i));
            end
        end;
        %fprintf(fid, '\n%s','! end of data');
        fclose(fid);
        fprintf('Output written in %s \n',filename);
        fprintf('Current directory is %s \n',pwd);
    end;
end
